import pytest

from eossr.api.zenodo import http_status


def test_ZenodoHTTPStatus():

    # good status, no error raised
    status = http_status.ZenodoHTTPStatus(200)
    assert status.code == 200
    assert status.name == 'OK'
    assert status.is_error() is False
    status.raise_error()  # does nothing

    # bad status, must raise an HTTPStatusError
    pytest.raises(
        http_status.HTTPStatusError,
        http_status.ZenodoHTTPStatus,
        400,
    )

    # wrong status code, must raise a ValueError
    pytest.raises(
        ValueError,
        http_status.ZenodoHTTPStatus,
        324324,
    )



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

import glob
import os
import shutil
import sys

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('..'))

# Notebooks
notebook_dir = '../examples/notebooks/'
os.makedirs('examples', exist_ok=True)
[shutil.copy(notebook_dir + file, 'examples') for file in os.listdir(notebook_dir) if file.endswith('.ipynb')]

# Snippets
snippets_doc_dir = 'snippets'
os.makedirs(snippets_doc_dir, exist_ok=True)
for filename in glob.glob('../examples/CI_code_snippets/*.md'):
    shutil.copy(filename, snippets_doc_dir)


# -- Project information -----------------------------------------------------

project = 'eossr'
copyright = '2021, ESCAPE WP3-OSSR developers and contributors'
author = 'Thomas Vuillaume & Enrique Garcia'

release = ''

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'nbsphinx',
    'sphinx.ext.viewcode',
    'myst_parser',
    'sphinx_multiversion',
    'sphinxcontrib.autoprogram',
    'sphinxcontrib.mermaid',
]


# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

html_sidebars = {
    '**': [
        'versioning.html',
    ],
}

# Parsers
# source_parsers = {
# }

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = [
    '.rst',
    '.md',
]


# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']


# nbsphinx
nbsphinx_allow_errors = False
nbsphinx_execute = 'auto'  # disable with 'never', force with 'always'

# sphinx-multiversion
# Whitelist pattern for tags (set to None to ignore all tags)
smv_tag_whitelist = r'^.*$'

# Whitelist pattern for branches (set to None to ignore all branches)
# smv_branch_whitelist = r'^.*$'
smv_branch_whitelist = 'master'

# Whitelist pattern for remotes (set to None to use local branches only)
smv_remote_whitelist = None

# Pattern for released versions
smv_released_pattern = r'.*v.*'

# Format for versioned output directories inside the build directory
smv_outputdir_format = '{ref.name}'

# Determines whether remote or local git branches/tags are preferred if their output dirs conflict
smv_prefer_remote_refs = False



#!/usr/bin/env python
# This script is meant to be used by the continuous integration of a git repository to upload its content to the OSSR


import argparse
import json
import tempfile
from copy import deepcopy
from pathlib import Path

from eossr.api.zenodo import Record, SimilarRecordError, ZenodoAPI
from eossr.metadata.codemeta2zenodo import converter
from eossr.metadata.zenodo import add_escape2020_community, add_escape2020_grant
from eossr.utils import zip_repository


def upload(
    zenodo_token,
    sandbox_flag,
    upload_directory,
    archive_name=None,
    record_id=None,
    erase_previous_files=True,
    force_new_record=False,
    publish=True,
    add_escape2020=False,
):
    """
    Zip the content of `upload_directory` and upload it into the OSSR.
    There must be a metadata file `codemeta.json` in the directory to be uploaded.

    The script will search for similar records previously uploaded by the same Zenodo user
    and will raise an error if it finds any and if a `record_id` was not provided or if `force_new_record==False`

    :param zenodo_token: str
        Personal access token to the (sandbox.)zenodo.org/api
    :param sandbox_flag: bold
        Set the Zenodo environment. True to use the sandbox, False to use Zenodo.
    :param upload_directory: str or Path
        Path to the directory whose content will be uploaded to the OSSR.
    :param archive_name: str or None
        Name of the zip archive. If None (default), the name of upload_directory is used.
    :param record_id: int or str
        Zenodo record-id of the record that is going to be updated.
        If no record_id is provided, a new record will be created in the OSSR.
    :param erase_previous_files: bool
        If True (default), it will erase the files of previous versions of the record before creating and updating the
        new version of the record. If False, it will not erase any file and old files will be included in the
        new version.
    :param force_new_record: bool
        If False (default), a new version of the `record_id` record will be created.
        If True, a new record - despite that it might already exists one - will be created.
    :param publish: bool
        If true, publish the record. Otherwise, the record is prepared but publication must be done manually. This
        is useful to check or discard the record before publication.
    :param add_escape2020: bool
        If True, adds ESCAPE project metadata: escape2020 community and ESCAPE grant number.

    :return: The `record_id` of the record created/uploaded
        `ZenodoAPI.upload_dir_content` answer
    """

    zenodo = ZenodoAPI(access_token=zenodo_token, sandbox=sandbox_flag)

    # Loads the metadata files if exists
    path_zenodo_file = zenodo.path_zenodo_file(upload_directory)
    path_codemeta_file = zenodo.path_codemeta_file(upload_directory)
    if path_zenodo_file.exists():
        print(f"Record metadata based on zenodo file {zenodo.path_zenodo_file}")
        with open(path_zenodo_file) as file:
            metadata = json.load(file)

    elif path_codemeta_file.exists():
        print(f"Record metadata based on codemeta file {path_codemeta_file}")
        with open(path_codemeta_file) as file:
            codemeta = json.load(file)
        metadata = converter(codemeta, add_escape2020=False)  # escape metadata can be added in both cases
    else:
        raise FileNotFoundError("No metadata provided")

    if add_escape2020:
        add_escape2020_community(metadata)
        add_escape2020_grant(metadata)

    metadata_for_check = {'metadata': deepcopy(metadata), 'id': 1}
    metadata_for_check['metadata']['doi'] = 1  # fake doi to create fake record
    record = Record(metadata_for_check)

    # Searches for similar records
    similar_records = zenodo.find_similar_records(record)
    if similar_records and not force_new_record and not record_id:
        raise SimilarRecordError(
            f"There are similar records in your own records: {similar_records}."
            "If you want to update an existing record, provide its record id to make a new version."
            "If you still want to make a new record, use --force-new-record."
        )

    with tempfile.TemporaryDirectory() as tmpdir:
        zip_filename = Path(upload_directory).absolute().name + '.zip' if archive_name is None else archive_name
        zip_repository(upload_directory, zip_filename=Path(tmpdir).joinpath(zip_filename))

        record_id = zenodo.upload_dir_content(
            tmpdir,
            record_id=record_id,
            metadata=metadata,
            erase_previous_files=erase_previous_files,
            publish=publish,
        )

    return record_id


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(
        description="Upload a directory to the OSSR as record. "
        "The directory must include a valid zenodo or codemeta file to be used"
        "as metadata source for the upload. "
        "If not record_id is passed, a new record is created. "
        "Otherwise, a new version of the existing record is created."
    )

    parser.add_argument(
        '--token', '-t', type=str, dest='zenodo_token', help='Personal access token to (sandbox)Zenodo', required=True
    )

    parser.add_argument(
        '--sandbox',
        '-s',
        action='store_true',
        help='Upload to Zenodo sandbox.',
        default=False,
    )

    parser.add_argument(
        '--input-dir',
        '-i',
        type=str,
        dest='input_directory',
        help='Path to the directory containing the files to upload.' 'All files will be uploaded.',
        required=True,
    )

    parser.add_argument(
        '--archive-name',
        type=str,
        default=None,
        dest='archive_name',
        help='Name of the upload zip archived. Default: name of in the input directory',
    )

    parser.add_argument(
        '--record_id',
        '-id',
        type=str,
        dest='record_id',
        help='record_id of the deposit that is going to be updated by a new version',
        default=None,
        required=False,
    )

    parser.add_argument(
        '--force-new-record',
        action='store_true',
        dest='force_new_record',
        help='Force the upload of a new record in case a similar record is found ' 'in the user existing ones',
    )

    parser.add_argument(
        '--no-publish',
        action='store_false',
        dest='publish',
        help='Optional tag to specify if the record will NOT be published. '
        'Useful for checking the record before publication of for CI purposes.',
    )

    parser.add_argument(
        '--add-escape2020',
        action='store_true',
        dest='add_escape2020',
        help='Add ESCAPE2020 community and grant number to the record.',
    )

    return parser


def main():

    parser = build_argparser()
    args = parser.parse_args()

    upload(
        args.zenodo_token,
        args.sandbox,
        args.input_directory,
        archive_name=args.archive_name,
        record_id=args.record_id,
        force_new_record=args.force_new_record,
        publish=args.publish,
        add_escape2020=args.add_escape2020,
    )


if __name__ == '__main__':
    main()



import warnings
from datetime import date

import jsonschema
import requests
from jsonref import replace_refs

from ..utils import write_json
from . import valid_semver

_zenodo_valid_licenses = None
_zenodo_deposit_schema = None


def _load_zenodo_deposit_schema():
    def update_dict_recursively(dict1, dict2):
        for key, value in dict2.items():
            if key in dict1 and isinstance(value, dict):
                update_dict_recursively(dict1[key], value)
            else:
                dict1[key] = value

    global _zenodo_deposit_schema
    zenodo_base_schema_url = "https://zenodo.org/schemas/records/base-v1.0.0.json"
    zenodo_deposit_schema_url = "https://zenodo.org/schemas/deposits/records/legacyrecord.json"
    zenodo_base_schema = replace_refs(requests.get(zenodo_base_schema_url).json())
    zenodo_deposit_schema = replace_refs(requests.get(zenodo_deposit_schema_url).json())
    _zenodo_deposit_schema = zenodo_base_schema
    update_dict_recursively(_zenodo_deposit_schema, zenodo_deposit_schema)
    _zenodo_deposit_schema['required'] = ['title', 'upload_type', 'creators', 'description']


class ZenodoMetadataValidationError(Exception):
    pass


def validate_zenodo_metadata_deposit(metadata):
    """
    Validate the zenodo metadata following the description from https://developers.zenodo.org/#representation before
    doing a deposit (upload). Note that once deposited, the metadata schema is different for a record.
    Raise a ValueError if the metadata is not valid.
    Raise warnings for some important metadata.

    Parameters:
    ----------
    metadata: dict
    """
    if "version" in metadata and not valid_semver(metadata["version"]):
        warnings.warn(f"Version {metadata['version']} does not follow the recommended format from semver.org.")

    if _zenodo_deposit_schema is None:
        _load_zenodo_deposit_schema()
    try:
        jsonschema.validate(metadata, _zenodo_deposit_schema)
    except jsonschema.exceptions.ValidationError as e:
        raise ZenodoMetadataValidationError(f"Zenodo metadata is not valid: {e.message}") from e

    # extra checks not in Zenodo schema
    if "publication_date" not in metadata:
        warnings.warn("Missing publication_date in the metadata.")
    else:
        try:
            date.fromisoformat(metadata["publication_date"])
        except ValueError as exc:
            raise ZenodoMetadataValidationError(
                f"Invalid publication_date {metadata['publication_date']}, not isoformat"
            ) from exc

    if "access_right" not in metadata:
        warnings.warn("Missing access_right in the metadata, defaults to open.")
    elif metadata["access_right"] in ["open", "embargoed"]:
        if "license" not in metadata:
            raise ZenodoMetadataValidationError("Missing license")
        elif not (valid_license(metadata["license"])):
            raise ZenodoMetadataValidationError(f"Invalid license {metadata['license']}")
    elif metadata["access_right"] == "restricted":
        if "embargo_date" not in metadata:
            raise ZenodoMetadataValidationError("Missing embargo_date")
        else:
            try:
                date.fromisoformat(metadata["embargo_date"])
            except ValueError as exc:
                raise ZenodoMetadataValidationError("Invalid embargo_date, not isoformat") from exc
    elif metadata["access_right"] == "restricted":
        if "access_conditions" not in metadata:
            raise ZenodoMetadataValidationError("Missing access_conditions")

    return None


def write_zenodo_metadata(zenodo_metadata, filename='.zenodo.json', overwrite=False, validate=True):
    """
    Write the zenodo metadata to a file.

    Parameters:
    ----------
    zenodo_metadata: dict
    filename: str
    """
    if validate:
        validate_zenodo_metadata_deposit(zenodo_metadata)
    write_json(zenodo_metadata, filename, overwrite=overwrite)


def add_escape2020_community(zenodo_metadata):
    """
    Add compulsory information to the Zenodo metadata:
     * zenodo community : ESCAPE2020

    :param zenodo_metadata: Zenodo metadata dictionary
    """
    if 'communities' not in zenodo_metadata:
        zenodo_metadata['communities'] = [{"identifier": "escape2020"}]
    elif all(community['identifier'] != 'escape2020' for community in zenodo_metadata['communities']):
        zenodo_metadata['communities'].append({"identifier": "escape2020"})


def add_escape2020_grant(zenodo_metadata):
    """
    Add compulsory information to the Zenodo metadata:
     * ESCAPE grant ID (zenodo syntax)

    :param zenodo_metadata: Zenodo metadata dictionary
    """
    if 'grants' not in zenodo_metadata:
        zenodo_metadata['grants'] = [{"id": "10.13039/501100000780::824064"}]
    elif all(grant['id'] != '10.13039/501100000780::824064' for grant in zenodo_metadata['grants']):
        zenodo_metadata['grants'].append({"id": "10.13039/501100000780::824064"})


def _load_valid_licenses():
    """
    Load the list of valid licenses from zenodo.org into global variable _zenodo_valid_licenses
    Note: this should use `eossr.api.zenodo.get_licenses` but importing here would lead to a circular import.
    A refactoring of the library is needed to avoid this.
    """
    global _zenodo_valid_licenses
    req = requests.get("https://zenodo.org/api/licenses", {'size': 1000})
    try:
        req.raise_for_status()
    except requests.exceptions.HTTPError as exc:
        raise requests.exceptions.HTTPError(f"Unable to get the list of licenses from Zenodo: {req.text}") from exc
    _zenodo_valid_licenses = [license["id"] for license in req.json()["hits"]["hits"]]


def valid_license(license_name):
    """
    Validate the license name.

    :param license_name: license name
    :return: True if the license is valid, False otherwise
    """
    global _zenodo_valid_licenses
    if _zenodo_valid_licenses is None:
        _load_valid_licenses()
    return license_name in _zenodo_valid_licenses



import json
from os.path import dirname, join, realpath

import pytest

from eossr.metadata import codemeta2zenodo

SAMPLES_DIR = join(dirname(realpath(__file__)), "samples")
ROOT_DIR = dirname(realpath("codemeta.json"))


codemeta_entries = [
    "@context",
    "@type",
    "name",
    "description",
    "license",
    "softwareVersion",
    "developmentStatus",
    "codeRepository",
    "dateCreated",
    "isAccessibleForFree",
    "isPartOf",
    "contIntegration",
    "issueTracker",
    "readme",
    "buildInstructions",
    "operatingSystem",
    "author",
    "contributor",
    "maintainer",
    "funder",
    "funding",
    "programmingLanguage",
    "softwareRequirements",
    "keywords",
    "downloadUrl",
    "dateModified",
    "datePublished",
    "runtimePlatform",
    "releaseNotes",
    "readme",
]

zenodo_entries = [
    "upload_type",
    "title",
    "description",
    "language",
    "access_right",
    "version",
    "keywords",
    "notes",
    "license",
    "publication_date",
    "creators",
    # #"communities",
    # #"grants",
    # #"contributors",
    # #"references",
]


@pytest.fixture()
def tmp_dir(tmp_path):
    test_dir = tmp_path
    test_dir.mkdir(exist_ok=True)
    return test_dir


def test_Codemeta2ZenodoController():
    codemeta_file = join(ROOT_DIR, "codemeta.json")
    converter = codemeta2zenodo.CodeMeta2ZenodoController.from_file(codemeta_file)
    assert converter.codemeta_data != {}
    assert all(key in converter.codemeta_data.keys() for key in codemeta_entries)

    converter.convert()
    assert converter.zenodo_data != {}
    assert converter.zenodo_data["language"] == "eng"
    assert converter.zenodo_data["access_right"] == "open"
    assert all(key in converter.zenodo_data.keys() for key in zenodo_entries)

    converter.add_escape2020_community()
    assert converter.zenodo_data["communities"] == [{"identifier": "escape2020"}]
    converter.add_escape2020_grant()
    assert converter.zenodo_data["grants"] == [{"id": "10.13039/501100000780::824064"}]


@pytest.mark.xfail
def test_codemeta_license_fails():
    codemeta_file = join(ROOT_DIR, "codemeta.json")
    converter = codemeta2zenodo.CodeMeta2ZenodoController.from_file(codemeta_file)
    converter.codemeta_data['license'] = "https://creativecommons.org/licenses/by/4.0/"
    converter.convert()


def test_add_author_metadata():

    with open(join(SAMPLES_DIR, "codemeta_contributors_sample.json")) as f:
        codemeta_metadata = json.load(f)
    zenodo_metadata = {}

    assert all(person in codemeta_metadata.keys() for person in codemeta2zenodo.codemeta_allowed_person_fields)

    for person in codemeta2zenodo.codemeta_allowed_person_fields:
        codemeta2zenodo.add_author_metadata(zenodo_metadata, codemeta_metadata[person], person)

    assert 'creators' in zenodo_metadata
    # 4 'creators' one repeated, should not be duplicated.
    # Maintainer and Contributor. Author and Creator are the same person
    assert len(zenodo_metadata['creators']) == 3

    assert 'contributors' in zenodo_metadata
    # Editor, Producer, Publisher, Provider and Sponsor
    assert len(zenodo_metadata['contributors']) == 5


def test_parse_person_schema_property():

    with open(join(SAMPLES_DIR, "codemeta_contributors_sample.json")) as f:
        codemeta_metadata = json.load(f)

    for person in codemeta2zenodo.codemeta_contributors_fields:
        zenodo_metadata = codemeta2zenodo.parse_person_schema_property(codemeta_metadata[person], person)
        if person == 'editor':
            assert zenodo_metadata['type'] == 'Editor'
        elif person == 'producer':
            assert zenodo_metadata['type'] == 'Producer'
        elif person == 'sponsor':
            assert zenodo_metadata['type'] == 'Sponsor'
        else:
            assert zenodo_metadata['type'] == 'Other'


# class TestConverting(unittest.TestCase):


def test_converter():
    with open(join(SAMPLES_DIR, "codemeta_sample1.json")) as file:
        codemeta = json.load(file)
    zenodo = codemeta2zenodo.converter(codemeta)
    assert zenodo['communities'][0]['identifier'] == 'escape2020'


def test_sample_file_conversion(tmp_dir):
    codemeta2zenodo.parse_codemeta_and_write_zenodo_metadata_file(join(SAMPLES_DIR, "codemeta_sample1.json"), tmp_dir)
    with open(tmp_dir.joinpath('.zenodo.json').name, 'r') as f:
        zen_meta = json.load(f)
    assert 'related_identifiers' in zen_meta
    assert "https://readme.com" in [ri['identifier'] for ri in zen_meta['related_identifiers']]


def test_root_codemeta_conversion(tmp_dir):
    codemeta2zenodo.parse_codemeta_and_write_zenodo_metadata_file(join(ROOT_DIR, "codemeta.json"), tmp_dir)
    with open(tmp_dir.joinpath('.zenodo.json').name, 'r') as f:
        json.load(f)


@pytest.mark.xfail(raises=KeyError)
def test_no_license_in_codemeta():
    converter = codemeta2zenodo.CodeMeta2ZenodoController({})
    converter.convert_license()


def test_convert_license():
    converter = codemeta2zenodo.CodeMeta2ZenodoController({})
    converter.codemeta_data['license'] = "https://spdx.org/licenses/MIT"
    converter.convert_license()
    assert converter.zenodo_data["license"] == "MIT"

    converter.codemeta_data["license"] = "Apache-2.0"
    converter.convert_license()
    assert converter.zenodo_data["license"] == "Apache-2.0"

    converter.codemeta_data["license"] = ["MIT", "https://spdx.org/licenses/BSD-3-Clause", "Apache-2.0"]
    converter.convert_license()
    assert converter.zenodo_data["license"] == "other-open"



#!/usr/bin/env python

import json
import warnings
from datetime import date
from pathlib import Path

from ..utils import spdx_licenses
from .zenodo import (
    add_escape2020_community,
    add_escape2020_grant,
    validate_zenodo_metadata_deposit,
    write_zenodo_metadata,
)

codemeta_creators_fields = ['author', 'creator', 'maintainer', 'contributor']
codemeta_contributors_fields = ['editor', 'producer', 'publisher', 'provider', 'sponsor']
codemeta_allowed_person_fields = codemeta_creators_fields + codemeta_contributors_fields


def parse_person_schema_property(person_property, contributor_field):
    """
    Parse the Person Schema property correctly

    Parameters:
    --------
    person_property: dict
        dictionary codemeta key with the a list or a single Person property
        item.
    contributor_field : str
        contributor type {'editor', 'producer', 'sponsor'} or publisher,
        although the last one can only happen if `upload_type` is publication

    Returns:
    --------
    zenodo_person: dict
        dictionary with the correct zenodo syntax for all {author, contributor,
        maintainer}.
    """
    zenodo_person = {}

    name = person_property['familyName']
    if 'givenName' in person_property:
        name += f', {person_property["givenName"]}'
    zenodo_person['name'] = name

    if "@id" in person_property:
        if 'orcid.org/' in person_property["@id"]:
            # reformat "https://orcid.org/0000-0002-5686-2078"
            zenodo_person['orcid'] = person_property["@id"].split('orcid.org/')[-1]
        else:
            zenodo_person['orcid'] = person_property["@id"]

    if "affiliation" in person_property:
        zenodo_person['affiliation'] = person_property['affiliation']['name']

    # Parse correctly the contributors
    if contributor_field in codemeta_contributors_fields:

        if contributor_field in ['provider', 'publisher']:
            zenodo_person['type'] = 'Other'
        else:
            # First letter of contributor type MUST be capitalized
            # (not for two words' contributor !)
            zenodo_person['type'] = contributor_field.title()

    return zenodo_person


def add_author_metadata(zenodo_file, codemt_person_entry, person_field):
    """
    Aux function to parse correctly all the authors, contributors and
    maintainers that can be found at the codemeta.json file

    zenodo_file: dict
        metadata dictionary with the zenodo syntax
    codemt_person_entry: list or dict
        metadata dictionary key field with the codemeta syntax
    person_field: str
        codemeta key field specifying creator {author, contributor, maintainer,
        creator}, or contributors {editor, sponsor, producer, project
        manager...}

    """
    full_contacts = {}

    # First create the full contact agenda by field
    if type(codemt_person_entry) is list:

        for person_property in codemt_person_entry:
            zenodo_person = parse_person_schema_property(person_property, person_field)
            # 'name' is the only key that MUST be contained in a
            # person_property at least
            full_contacts[zenodo_person['name']] = zenodo_person
    else:
        zenodo_person = parse_person_schema_property(codemt_person_entry, person_field)
        full_contacts[zenodo_person['name']] = zenodo_person

    # then save each person by field and avoid duplicates
    for person, value in full_contacts.items():

        if person_field in codemeta_creators_fields:

            # Contributors and maintainers in the same zenodo key
            if 'creators' not in zenodo_file:
                zenodo_file['creators'] = []

            if value not in zenodo_file['creators']:
                zenodo_file['creators'].append(full_contacts[person])
        elif person_field in codemeta_contributors_fields:

            if 'contributors' not in zenodo_file:
                zenodo_file['contributors'] = []

            if full_contacts[person] not in zenodo_file['contributors']:
                zenodo_file['contributors'].append(full_contacts[person])


def find_matching_metadata(codemeta_json):
    """
    Please note that the following fields are ASSUMED. If they are not
    correct, change them, or contact us otherwise.
        * "access_right": "open"
        * "language": "eng"

    param codemeta_json: dict
        already parsed dictionary containing the metadata of the codemeta.json
        file

    Returns:
    --------
    metadata_zenodo : dict
        dictionary cotaining the metadata information found at the
        codemeta.json file but written using the Zenodo syntax.
    """

    # All the 'person type' allowed in the CodeMeta schema are listed in the
    # 'codemeta_allowed_person_fields' list.  However, the Zenodo schema
    # does not accept certain codemeta 'person type' properties; like
    # publisher and provider, nor all the extended schema.org 'person type'
    # (actor, director, member, performer ...).
    # The crosswalk will be limited to the 'codemeta_allowed_person_fields'
    # list.
    if not codemeta_json['@context'] == 'https://doi.org/10.5063/schema/codemeta-2.0':
        raise ValueError(
            f"The converter has been developed for codemeta-2.0 "
            f"but the provided context is {codemeta_json['@context']}."
        )

    def append_related_identifiers(metadata_zenodo: dict, meta_dict: dict):
        if 'related_identifiers' in metadata_zenodo:
            if not isinstance(metadata_zenodo['related_identifiers'], list):
                raise TypeError(
                    f"metadata_zenodo['related_identifiers'] should be a list, "
                    f"but is {type(metadata_zenodo['related_identifiers'])}"
                )
            metadata_zenodo['related_identifiers'].append(meta_dict)
        else:
            metadata_zenodo['related_identifiers'] = [meta_dict]
        return metadata_zenodo

    metadata_zenodo = {'language': 'eng', 'access_right': 'open'}

    if "@type" not in codemeta_json or codemeta_json["@type"] == "SoftwareSourceCode":
        metadata_zenodo['upload_type'] = 'software'
    else:
        raise ValueError(
            f"CodeMeta schema has been developed for software, '@type' key must be 'SoftwareSourceCode' "
            f"but is {metadata_zenodo['upload_type']}"
        )

    if 'name' in codemeta_json:
        metadata_zenodo['title'] = codemeta_json['name']

    if 'description' in codemeta_json:
        metadata_zenodo['description'] = codemeta_json['description']

    if (
        'softwareVersion' in codemeta_json
        and 'version' in codemeta_json
        and codemeta_json['softwareVersion'] != codemeta_json['version']
    ):
        raise ValueError('`softwareVersion` and `version` have different values')
    if 'softwareVersion' in codemeta_json:
        metadata_zenodo['version'] = str(codemeta_json['softwareVersion'])
    elif 'version' in codemeta_json:
        metadata_zenodo['version'] = str(codemeta_json['version'])
    else:
        raise TypeError("A version must be provided in codemeta, either with `version` or `softwareVersion`")

    if 'keywords' in codemeta_json:
        if type(codemeta_json['keywords']) == list:
            metadata_zenodo['keywords'] = codemeta_json['keywords']
        else:
            metadata_zenodo['keywords'] = [codemeta_json['keywords']]

    if 'releaseNotes' in codemeta_json:
        metadata_zenodo['notes'] = "Release Notes: " + codemeta_json['releaseNotes']

    if 'citation' in codemeta_json:
        metadata_zenodo['references'] = codemeta_json['citation']

    if 'datePublished' in codemeta_json:
        metadata_zenodo['publication_date'] = codemeta_json['datePublished']
    else:
        metadata_zenodo['publication_date'] = str(date.today())

    for person_type in codemeta_allowed_person_fields:
        if person_type in codemeta_json:
            add_author_metadata(metadata_zenodo, codemeta_json[person_type], person_field=person_type)

    if 'codeRepository' in codemeta_json:
        meta_dict = {
            "scheme": "url",
            "identifier": codemeta_json['codeRepository'],
            "relation": "isDerivedFrom",
            "resource_type": metadata_zenodo['upload_type'],
        }
        metadata_zenodo = append_related_identifiers(metadata_zenodo, meta_dict)

    if 'readme' in codemeta_json:
        meta_dict = {
            "scheme": "url",
            "identifier": codemeta_json['readme'],
            "relation": "isDocumentedBy",
            "resource_type": "publication-softwaredocumentation",
        }
        metadata_zenodo = append_related_identifiers(metadata_zenodo, meta_dict)

    return metadata_zenodo


class CodeMeta2ZenodoController(object):
    """Control the conversion of a codemeta file to a zenodo file"""

    def __init__(self, codemeta_dict):
        assert isinstance(codemeta_dict, dict)
        self.codemeta_data = codemeta_dict
        self.zenodo_data = {}

    @classmethod
    def from_file(cls, codemeta_filename):
        """Load `codemeta_filename` into the converter"""
        with open(codemeta_filename) as infile:
            controller = cls(json.load(infile))
        return controller

    def convert_license(self):
        def strip_spdx_url(license_url):
            return license_url.replace("https://spdx.org/licenses/", "")

        if 'license' not in self.codemeta_data:
            raise KeyError("Missing license in codemeta file")
        record_license = self.codemeta_data.get('license')
        if isinstance(record_license, list):
            spdx = spdx_licenses()
            licenses_ids = [strip_spdx_url(license_url) for license_url in record_license]
            if all([license_id in spdx for license_id in licenses_ids]):
                self.zenodo_data['license'] = 'other-open'
            else:
                warnings.warn(
                    "A list of licenses has been provided but some of them are not SPDX licenses." "Using 'other-open'."
                )
                self.zenodo_data['license'] = 'other-closed'
        else:
            self.zenodo_data['license'] = strip_spdx_url(record_license)

    def convert(self, validate=True):
        """Convert data over to zenodo format"""
        self.zenodo_data = find_matching_metadata(self.codemeta_data)
        self.convert_license()
        if validate:
            self.validate()

    def validate(self):
        """
        Validate the zenodo data.
        """
        validate_zenodo_metadata_deposit(self.zenodo_data)

    def add_escape2020_community(self):
        """
        Add compulsory information to the .zenodo.json file:
         * zenodo community : ESCAPE2020
        """
        add_escape2020_community(self.zenodo_data)

    def add_escape2020_grant(self):
        """
        Add compulsory information to the .zenodo.json file:
         * ESCAPE grant ID (zenodo syntax)
        """
        add_escape2020_grant(self.zenodo_data)

    def write_zenodo(self, zenodo_filename='.zenodo.json', overwrite=False, validate=True):
        """Write `zenodo_filename` after conversion"""
        write_zenodo_metadata(self.zenodo_data, zenodo_filename, overwrite=overwrite, validate=validate)


def converter(codemeta_dict, add_escape2020=True):
    """
    Convert codemeta metadata into zenodo metadata

    :param codemeta_dict: dict
    :param add_escape2020: bool
        if True, add escape2020 community and grant
    :return: dict
        zenodo metadata
    """
    meta_converter = CodeMeta2ZenodoController(codemeta_dict)
    meta_converter.convert()
    if add_escape2020:
        meta_converter.add_escape2020_community()
        meta_converter.add_escape2020_grant()
    return meta_converter.zenodo_data


def parse_codemeta_and_write_zenodo_metadata_file(codemeta_filename, outdir, add_escape2020=True, overwrite=True):
    """
    Reads the codemeta.json file and creates a new `.zenodo.json` file in outdir.
    This file contains the same information that in the codemeta.json file but following the zenodo metadata schema.

    codemeta_filename: str or Path
        path to the codemeta.json file
    outdir: str or Path
        path to the outdir where the file `.zenodo.json` will be created
    add_escape2020: bool
        adds escape2020 metadata in zenodo metadata file
    overwrite: bool
        overwrite existing `.zendoo.json` file in `outdir`
    """
    meta_converter = CodeMeta2ZenodoController.from_file(codemeta_filename)
    meta_converter.convert()
    if add_escape2020:
        meta_converter.add_escape2020_community()
        meta_converter.add_escape2020_grant()
    outfile = Path(outdir).joinpath('.zenodo.json')
    meta_converter.write_zenodo(outfile.name, overwrite=overwrite)



#!/usr/bin/env python

import argparse

from eossr.utils import zip_repository


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(
        description="Zip the content of a directory (a git project is expected). "
        "`.git` subdirectories in the target directory will be excluded"
    )

    parser.add_argument(
        '--directory', '-d', type=str, dest='directory', help='Path to the directory to be zipped.', required=True
    )

    parser.add_argument(
        '--name_zip', '-n', type=str, dest='name_zip', help='Zip filename. DEFAULT: directory (basename)', default=None
    )
    return parser


def main():

    parser = build_argparser()
    args = parser.parse_args()
    zip_repository(
        args.directory,
        args.name_zip,
    )


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import argparse

from eossr.api.zenodo import ZenodoAPI


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(description="Test the connection to zenodo and all the stages of a new upload.")

    parser.add_argument(
        '--token', '-t', type=str, dest='zenodo_token', help='Personal access token to (sandbox)Zenodo', required=True
    )

    parser.add_argument(
        '--sandbox',
        '-s',
        action='store_true',
        help='Use Zenodo sandbox.',
    )

    parser.add_argument(
        '--project_dir',
        '-p',
        action='store',
        dest='project_dir',
        help='Path to the root directory of the directory to be uploaded. ' 'DEFAULT; assumed to be on it, i.e., "./"',
        default='./',
    )
    return parser


def main():
    # Required arguments

    args = build_argparser().parse_args()

    zenodo = ZenodoAPI(access_token=args.zenodo_token, sandbox=args.sandbox)
    zenodo.check_upload_to_zenodo(args.project_dir)


if __name__ == '__main__':
    main()



from eossr import api


def test_get_ossr_records():
    ossr_records = api.get_ossr_records(all_versions=True)
    assert len(ossr_records) >= 12  # number of records October 01, 2021
    all_ids = [rec.data['id'] for rec in ossr_records]
    assert 5524913 in all_ids  # id of the version v0.2 of the eossr



# this is adapted from https://github.com/astropy/astropy/blob/master/astropy/version.py
# see https://github.com/astropy/astropy/pull/10774 for a discussion on why this needed.
import warnings

try:
    try:
        from ._dev_version import version
    except ImportError:
        from ._version import __version__ as version
except Exception as e:
    warnings.warn(
        "Could not determine eossr version; this indicates a broken installation."
        " Install eossr from PyPI, a local git repository or using "
        " pip install git+https://gitlab.in2p3.fr/escape2020/wp3/eossr@VERSION "
        " Installing gitlab's autogenerated source release tarballs "
        " does not include version information and should be avoided."
        f"Exception Error: {e}"
    )
    version = "0.0.0"

__version__ = version



#!/usr/bin/env python
import json
import os
import shutil
import tempfile
import unittest
from pathlib import Path

import pytest
import requests

from eossr import ROOT_DIR
from eossr.api import zenodo
from eossr.api.zenodo import Record, ZenodoAPI, get_record, get_zenodo_records
from eossr.api.zenodo.http_status import HTTPStatusError
from eossr.api.zenodo.zenodo import is_live, query_record

# test deactivated at the moment
# eossr_test_lib_id = 930570  # test library in sandbox (owner: T. Vuillaume)


def test_is_live():
    assert is_live()
    assert is_live(sandbox=True)


class TestZenodoApiSandbox(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestZenodoApiSandbox, self).__init__(*args, **kwargs)
        self.token = 'FakeToken'
        self.zenodo = ZenodoAPI(access_token=self.token, sandbox=True)

        # getting all records for tests purposes
        self.zenodo.parameters['size'] = 1000
        self.zenodo.parameters['all_versions'] = True

    def test_initialization_sandbox(self):
        assert isinstance(self.zenodo, ZenodoAPI)
        assert self.zenodo.api_url == 'https://sandbox.zenodo.org/api'
        assert self.zenodo.access_token == self.token
        assert self.zenodo.path_codemeta_file(ROOT_DIR) == ROOT_DIR.joinpath('codemeta.json')
        assert self.zenodo.path_zenodo_file(ROOT_DIR) == ROOT_DIR.joinpath('.zenodo.json')

    def test_query_community_entries(self):
        community_entries = self.zenodo.query_community_records('escape2020')
        assert isinstance(community_entries, requests.models.Response)


class TestZenodoAPINoToken(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestZenodoAPINoToken, self).__init__(*args, **kwargs)
        self.token = ''
        self.zenodo = ZenodoAPI(access_token=self.token, sandbox=False)

    def test_initialization(self):
        assert isinstance(self.zenodo, ZenodoAPI)
        assert self.zenodo.api_url == 'https://zenodo.org/api'
        assert self.zenodo.access_token == self.token

    @pytest.mark.xfail(raises=ValueError)
    def test_raise_token_status(self):
        # A value error should be raised as no valid token was provided
        self.zenodo._raise_token_status()


@pytest.mark.skipif(os.getenv('SANDBOX_ZENODO_TOKEN') is None, reason="SANDBOX_ZENODO_TOKEN not defined")
class TestZenodoAPITokenSandbox(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestZenodoAPITokenSandbox, self).__init__(*args, **kwargs)
        self.token = os.getenv('SANDBOX_ZENODO_TOKEN')
        self.zenodo = ZenodoAPI(access_token=self.token, sandbox=True)

    def test_init(self):
        assert self.zenodo.access_token == os.getenv('SANDBOX_ZENODO_TOKEN')

    def test_raise_token_status(self):
        self.zenodo._raise_token_status()

    def test_query_user_deposits(self):
        self.zenodo.query_user_deposits()

    def test_create_erase_new_deposit(self):
        create_new_deposit = self.zenodo.create_new_deposit()
        assert isinstance(create_new_deposit, requests.models.Response)
        record_id = create_new_deposit.json()['id']
        erase_deposit = self.zenodo.erase_deposit(record_id)
        assert isinstance(erase_deposit, requests.models.Response)

    def test_upload_package(self):
        # TODO: implement pytest unit test and fixture
        # prepare upload in a tmpdir
        with tempfile.TemporaryDirectory() as tmpdirname:
            print(f"tmpdir {tmpdirname}")
            shutil.copy(ROOT_DIR.joinpath('codemeta.json'), tmpdirname)
            _, filename = tempfile.mkstemp(dir=tmpdirname)
            Path(filename).write_text('Hello from eossr unit tests')

            # create new record
            new_record_id = self.zenodo.upload_dir_content(tmpdirname, publish=False)
            self.zenodo.erase_deposit(new_record_id)
            print(f"{new_record_id} created and deleted")

            # update existing record
            # Test deactivated on July 2022 as making new versions of records seems not possible anymore on sandbox,
            # and I am reluctant to do unit tests directly on Zenodo.
            # new_record_id = self.zenodo.upload_dir_content(tmpdirname, record_id=eossr_test_lib_id, publish=False)
            # self.zenodo.erase_entry(new_record_id)
            # print(f"{new_record_id} created and deleted")

    @pytest.mark.skipif(
        os.getenv('SANDBOX_ZENODO_TOKEN_GARCIA') is None, reason="SANDBOX_ZENODO_TOKEN_GARCIA not defined"
    )
    def test_pending_request(self):
        zk = ZenodoAPI(os.getenv('SANDBOX_ZENODO_TOKEN_GARCIA'), sandbox=True)
        record_id = 970583
        from eossr.api.ossr import escape_community

        # Add to escape2020 community and test request
        meta = {'communities': [{'identifier': escape_community}]}
        zk.update_record_metadata(record_id, meta)

        # Well.... we should test this but sandbox is sometimes too slow and make this test fail unpredictably
        # Give some time to zenodo to process the request and update its database
        # time.sleep(10)
        # records = self.zenodo.get_community_pending_requests('escape2020')
        # assert record_id in [rec.id for rec in records]

        # TODO: After accept_pending_request is implemented
        # self.zenodo.accept_pending_request(escape_community, record_id)
        # records = get_zenodo_records(community=escape_community, sandbox=True, size=4)
        # assert record_id in [rec.id for rec in records]

        # Remove from community
        meta = {'communities': []}
        zk.update_record_metadata(record_id, meta)
        records = get_zenodo_records(community=escape_community, sandbox=True, size=4)
        assert record_id not in [rec.id for rec in records]

    def test_get_community_pending_requests(self):
        z = self.zenodo
        records = z.get_community_pending_requests('escape2020')
        records_ids = [rec.id for rec in records]
        assert 532458 in records_ids  # pending request in escape2020 sandbox community - 2021-11-28


@pytest.mark.skipif(os.getenv('ZENODO_TOKEN') is None, reason="ZENODO_TOKEN not defined")
class TestZenodoAPIToken(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestZenodoAPIToken, self).__init__(*args, **kwargs)
        self.token = os.getenv('ZENODO_TOKEN')
        self.zenodo = ZenodoAPI(access_token=self.token, sandbox=False)

    def test_get_user_records(self):
        records = self.zenodo.get_user_records()
        assert isinstance(records[0], Record)

    def test_find_similar_records_sandbox(self):
        self.zenodo.parameters.update({'size': 100})
        existing_record = Record.from_id(939075, sandbox=True)  # One of the copies of eossr
        assert len(self.zenodo.find_similar_records(existing_record)) > 0
        not_existing_record = Record.from_id(767507, sandbox=True)  # Sandbox E.GARCIA ZenodoCI_vTest Records
        assert self.zenodo.find_similar_records(not_existing_record) == []


def test_get_zenodo_records():
    zenodo_records = get_zenodo_records('ESCAPE template project', all_versions=True)
    assert len(zenodo_records) > 1
    all_dois = [r.data['doi'] for r in zenodo_records]
    assert '10.5281/zenodo.4923992' in all_dois


@pytest.mark.xfail(raises=HTTPStatusError)
def test_get_record_42():
    get_record(42)


def test_query_record_941144():
    # unsubmitted record in T. Vuillaume's sandbox - for test purposes
    answer = query_record(941144, sandbox=True)
    assert answer.status_code == 404


@pytest.mark.xfail(raises=HTTPStatusError)
def test_get_record_941144():
    get_record(941144, sandbox=True)


@pytest.fixture
def record_4923992():
    return get_record(4923992)


def test_record(record_4923992):
    assert record_4923992.data['conceptdoi'] == '10.5281/zenodo.3572654'
    record_4923992.print_info()
    codemeta = record_4923992.get_codemeta()
    assert isinstance(codemeta, dict)
    assert codemeta['name'] == 'ESCAPE template project'
    record_4923992.get_mybinder_url()


def test_get_record_sandbox():
    record = get_record(520735, sandbox=True)
    assert record.data['doi'] == '10.5072/zenodo.520735'


def test_write_record_zenodo(record_4923992, tmpdir):
    record_4923992._write_zenodo_deposit(filename=tmpdir / '.zenodo.json', validate=False)
    with open(tmpdir / '.zenodo.json') as file:
        json_dict = json.load(file)
    assert json_dict['title'] == 'ESCAPE template project'
    assert json_dict['version'] == 'v2.2'


def test_search_records():
    records = zenodo.search_records('conceptrecid:5524912')
    assert records[0]['metadata']['title'] == 'eossr'


def test_search_funders():
    funders = zenodo.search_funders('name:European+Commission')
    assert len(funders) > 1


def test_search_grants():
    grants = zenodo.search_grants('code:824064')
    assert len(grants) == 1
    assert grants[0]['metadata']['acronym'] == 'ESCAPE'


def test_search_license():
    licenses = zenodo.search_licenses('id:MIT')
    assert licenses[0]['metadata']['title'] == 'MIT License'


def test_search_communities():
    communities = zenodo.search_communities('escape2020')
    assert communities[0]['title'] == 'ESCAPE OSSR'


def test_get_associated_versions():
    record = Record.from_id(4786641)  # ZenodoCI deprecated lib. A single version
    versions = record.get_associated_versions()
    assert len(versions) == 1
    assert list(versions)[0] == record.id  # itself
    eossr_record = Record.from_id(6352039)
    eossr_record_versions = eossr_record.get_associated_versions()
    assert len(eossr_record_versions) >= 7  # Seven versions, to date 21/03/2022
    for recid, version in eossr_record_versions.items():
        assert eossr_record.data['conceptrecid'] == Record.from_id(recid).data['conceptrecid']
    assert eossr_record_versions[5524913] == 'v0.2'  # ID of eOSSR version v0.2


@pytest.mark.xfail(raises=FileNotFoundError)
def test_get_codemeta_fail():
    record = Record.from_id(3734091)
    record.get_codemeta()


def test_get_supported_licenses():
    zenodo_licenses = zenodo.get_supported_licenses()
    assert isinstance(zenodo_licenses, list)
    assert 'MIT' in zenodo_licenses
    assert 'Apache-2.0' in zenodo_licenses
    assert "Apache License x." not in zenodo_licenses


def test_download_record():
    with tempfile.TemporaryDirectory() as tmpdir:
        record = Record.from_id(3743490)
        record.download(tmpdir)
        assert os.path.exists(f'{tmpdir}/template_project_escape-v1.1.zip')



from copy import deepcopy

import pytest

from eossr.metadata import zenodo

meta_base = {
    "title": "eOSSR unit test",
    "description": "eOSSR unit tests.",
    "version": "1.7.0",
    "creators": [{"affiliation": "The best affiliation", "name": "Rick"}],
    "access_right": "open",
    "upload_type": "software",
    "license": "MIT",
}


def test_zenodo_metadata():
    meta = meta_base
    zenodo.validate_zenodo_metadata_deposit(meta)


def test_zenodo_metadata_publication_type():
    meta = deepcopy(meta_base)
    meta['upload_type'] = "publication"
    for publication_type in [
        "annotationcollection",
        "article",
        "book",
        "section",
        "conferencepaper",
        "datamanagementplan",
        "article",
        "patent",
        "preprint",
        "deliverable",
        "milestone",
        "proposal",
        "report",
        "softwaredocumentation",
        "taxonomictreatment",
        "technicalnote",
        "thesis",
        "workingpaper",
        "other",
    ]:
        meta["publication_type"] = publication_type
        zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_publication_type_fail():
    meta = deepcopy(meta_base)
    meta['upload_type'] = "publication"
    meta['publication_type'] = "invalid"
    zenodo.validate_zenodo_metadata_deposit(meta)


def test_zenodo_metadata_image_type():
    meta = deepcopy(meta_base)
    meta['upload_type'] = "image"
    for publication_type in ["figure", "plot", "drawing", "diagram", "photo", "other"]:
        meta["image_type"] = publication_type
        zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_access_right_fail():
    meta = deepcopy(meta_base)
    meta["access_right"] = "invalid"
    zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_image_type_fail():
    meta = deepcopy(meta_base)
    meta['upload_type'] = "publication"
    meta['publication_type'] = "invalid"
    zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_creator_noname():
    meta = deepcopy(meta_base)
    meta['creators'] = [{'affiliation': 'The best affiliation'}]
    zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_access_right_license():
    meta = deepcopy(meta_base)
    meta['access_right'] = 'open'
    meta.pop('license')
    zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_embargo_date():
    meta = deepcopy(meta_base)
    meta['access_right'] = 'restricted'
    zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_access_conditions():
    meta = deepcopy(meta_base)
    meta['access_right'] = 'restricted'
    zenodo.validate_zenodo_metadata_deposit(meta)


@pytest.mark.xfail(raises=zenodo.ZenodoMetadataValidationError)
def test_zenodo_metadata_fail():
    meta = deepcopy(meta_base)
    meta.pop("description")
    zenodo.validate_zenodo_metadata_deposit(meta)


def test_add_escape2020_community():
    meta = deepcopy(meta_base)
    zenodo.add_escape2020_community(meta)
    assert meta["communities"] == [{"identifier": "escape2020"}]

    zenodo.add_escape2020_community(meta)
    assert meta["communities"] == [{"identifier": "escape2020"}]

    meta["communities"] = [{"identifier": "escape2021"}]
    zenodo.add_escape2020_community(meta)
    assert meta["communities"] == [
        {"identifier": "escape2021"},
        {"identifier": "escape2020"},
    ]
    zenodo.validate_zenodo_metadata_deposit(meta)


def test_add_escape2020_grant():
    meta = deepcopy(meta_base)
    zenodo.add_escape2020_grant(meta)
    assert meta["grants"] == [{"id": "10.13039/501100000780::824064"}]

    meta["grants"] = [{"id": "10.13039/501100000780::824064"}]
    zenodo.add_escape2020_grant(meta)
    assert meta["grants"] == [{"id": "10.13039/501100000780::824064"}]

    meta["grants"] = [{"id": "another grant"}]
    zenodo.add_escape2020_grant(meta)
    assert meta["grants"] == [{"id": "another grant"}, {"id": "10.13039/501100000780::824064"}]
    zenodo.validate_zenodo_metadata_deposit(meta)


def test_valid_license():
    assert zenodo.valid_license("MIT")
    assert ~zenodo.valid_license("MIT2")



#!/usr/bin/env python

__all__ = ['ZenodoHTTPStatus']


class HTTPStatusError(Exception):
    pass


class ZenodoHTTPStatus:
    status_codes = {
        200: {
            'name': 'OK',
            'description': 'Request succeeded. Response included. Usually sent for GET/PUT/PATCH requests.',
        },
        201: {
            'name': 'Created',
            'description': 'Request succeeded. Response included. Usually sent for POST requests.',
        },
        202: {
            'name': 'Accepted',
            'description': 'Request succeeded. Response included. Usually sent for POST requests,'
            'where background processing is needed to fulfill the request.',
        },
        204: {
            'name': 'No Content',
            'description': 'Request succeeded. No response included. Usually sent for DELETE requests',
        },
        400: {'name': 'Bad Request', 'description': 'Request failed. Error response included.'},
        401: {
            'name': 'Unauthorized',
            'description': 'Request failed, due to an invalid access token. Error response included.',
        },
        403: {
            'name': 'Forbidden',
            'description': 'Request failed, due to missing authorization'
            '(e.g. deleting an already submitted upload or missing scopes for your access token).'
            'Error response included.',
        },
        404: {
            'name': 'Not Found',
            'description': 'Request failed, due to the resource not being found. Error response included.',
        },
        405: {
            'name': 'Method Not Allowed',
            'description': 'Request failed, due to unsupported HTTP method. Error response included.',
        },
        409: {
            'name': 'Conflict',
            'description': 'Request failed, due to the current state of the resource'
            '(e.g. edit a deopsition which is not fully integrated).'
            'Error response included.',
        },
        410: {
            'name': 'Deleted',
            'description': 'PID has been deleted',
        },
        415: {
            'name': 'Unsupported Media Type',
            'description': 'Request failed, due to missing or invalid request header Content-Type.'
            'Error response included.',
        },
        429: {
            'name': 'Too Many Requests',
            'description': 'Request failed, due to rate limiting.' 'Error response included.',
        },
        500: {
            'name': 'Internal Server Error',
            'description': 'Request failed, due to an internal server error. Error response NOT included.'
            'Don’t worry, Zenodo admins have been notified and will be dealing with the problem ASAP.',
        },
    }

    def __init__(self, code: int, json=None):
        if code not in self.status_codes.keys():
            raise ValueError("Unkown status code")
        self.code = code
        self.json = json
        if self.is_error():
            self.raise_error()

    def __str__(self):
        return f"HTTP Status Code: {self.code} - {self.name}.\n{self.description}"

    def __repr__(self):
        return f"ZenodoHttpStatus({self.code})"

    def is_error(self):
        return self.code >= 400

    @property
    def name(self):
        return self.status_codes[self.code]['name']

    @property
    def description(self):
        return self.status_codes[self.code]['description']

    def raise_error(self):
        """
        Raise an HTTPStatusError if the status code corresponds to an error
        """
        if self.is_error():
            msg = f"{self.__str__()}"
            if self.json:
                msg += f"\n{self.json}"
            raise HTTPStatusError(msg)



#!/usr/bin/env python

import requests

from .zenodo import ZenodoAPI, get_zenodo_records, zenodo_api_url

__all__ = [
    'get_ossr_records',
    'get_ossr_pending_requests',
]

escape_community = 'escape2020'


def get_ossr_records(search='', sandbox=False, **kwargs):
    """
    Search the OSSR for records whose names or descriptions include the provided string `search`.
    The default record type is 'software' or 'record'.
    Function rewritten from pyzenodo3 (https://github.com/space-physics/pyzenodo3)

    :param search: string
        A string to refine the search in the OSSR. The default will search for all records in the OSSR.
    :param sandbox: bool
        Indicates the use of sandbox zenodo or not.
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
        - size: int
        Number of results to return. Default = 100
        - all_versions: int
        Show (1) or hide (0) all versions of records
        - type: string or list[string]
        Default: ['software', 'dataset']
        Records of the specified type (Publication, Poster, Presentation, Software, ...).
        A logical OR is applied in case of a list
        - keywords: string or list[string]
        Records with the specified keywords. A logical OR is applied in case of a list
        - file_type: string or list[string]
        Records from the specified keywords. A logical OR is applied in case of a list

    :return: [Record]
    """

    # make sure we find all OSSR records without limit on the number
    params = kwargs
    params['communities'] = escape_community
    r = requests.get(zenodo_api_url + '/records', params=params)
    number_of_ossr_entries = r.json()['aggregations']['access_right']['buckets'][0]['doc_count']
    kwargs['size'] = number_of_ossr_entries

    # if another community is specified, a logical OR is applied by zenodo API,
    # thus potentially finding entries that are not part of escape2020
    # ruling out that possibility at the moment
    if 'communities' in kwargs and kwargs['communities'] != escape_community:
        raise NotImplementedError(
            "Searching in another community will search outside of the OSSR"
            "Use `eossr.api.zenodo.get_zenodo_records` to do so"
        )
    kwargs['communities'] = escape_community

    # OSSR is limited to software and datasets
    kwargs.setdefault('type', ['software', 'dataset'])

    return get_zenodo_records(search, sandbox=sandbox, **kwargs)


def get_ossr_pending_requests(**params):
    """
    Get a list of records that have been requested to be added to the OSSR.

    :param params: dict
        Parameters for the request. Override the class parameters.
    :return:
    """
    zen = ZenodoAPI()
    return zen.get_community_pending_requests(escape_community, **params)



import json
import os
from datetime import date
from pathlib import Path
from zipfile import ZipFile

import remotezip
import requests
import urllib3.exceptions
from markdown import markdown

from . import ROOT_DIR

__all__ = [
    'ZipUrl',
    'get_codemeta_from_zipurl',
    'zip_repository',
    'update_codemeta',
]


class ZipUrl(remotezip.RemoteZip, ZipFile):
    def __init__(self, url, initial_buffer_size=48 * 1024, **kwargs):
        # Zenodo imposes a limit on the zipfile to be read.
        # Some tests show that a limit with 48*1024 works fine.
        try:
            super().__init__(url, initial_buffer_size=initial_buffer_size, **kwargs)
        except requests.packages.urllib3.exceptions.ProtocolError as e:
            raise urllib3.exceptions.ProtocolError(
                f"{str(e)}\nTry lowering the initial buffer size for this zipfile"
            ) from e

    def find_files(self, filename):
        """
        return the path of files in the archive matching `filename`

        :param filename: string
        :return: list[str]
        """
        matching_files = [f for f in self.namelist() if Path(f).name == filename]
        if len(matching_files) == 0:
            raise FileNotFoundError(f"No file named {filename} in {self.url}")
        else:
            return matching_files

    def get_codemeta(self):
        codemeta_paths = self.find_files('codemeta.json')
        # if there are more than one codemeta file in the archive, we consider the one in the root directory, hence the
        # one with the shortest path
        codemeta_path = min(codemeta_paths, key=len)
        with self.open(codemeta_path) as file:
            codemeta = json.load(file)
        return codemeta


def get_codemeta_from_zipurl(url, **zipurl_kwargs):
    """
    Extract and reads codemeta metadata from a zip url.
    A codemeta.json file must be present in the zip archive.

    Parameters
    ----------
    url: string
        url to a zip file
    zipurl_kwargs: dictionnary
        metadata in the codemeta.json file in the zip archive

    Returns
    -------
    dict
    """
    zipurl_kwargs.setdefault('initial_buffer_size', 100)
    zipurl = ZipUrl(url, **zipurl_kwargs)
    return zipurl.get_codemeta()


def zip_repository(repository_path, zip_filename=None, overwrite=True):
    """
    Zip the content of `repository_path`
    `.git` subdirectories in the target directory will be excluded

    :param repository_path: str or Path
        Path to the directory to be zipped
    :param zip_filename: str
        Zip filename name, used to name the zip file. If None, the zip will be named as the directory provided.
    :param overwrite: bool
        True to overwrite existing zip archive

    :return: zip_filename: path to the zip archive
    """
    # prepare zip archive
    directory = Path(repository_path).resolve()
    zip_filename = f'{directory.absolute().name}.zip' if zip_filename is None else zip_filename
    if Path(zip_filename).exists() and not overwrite:
        raise FileExistsError(f"{zip_filename} exists. Set overwrite=True")

    print(f" * Zipping the content of {directory.absolute()} into {zip_filename}")

    with ZipFile(zip_filename, 'w') as zipObj:
        for folder_name, subfolders, filenames in os.walk(directory):
            # don't zip .git/ content nor the .git dir itself
            if '.git/' in folder_name or folder_name.endswith('.git'):
                continue
            # we want the relative path only inside the archive
            relpath = os.path.relpath(folder_name, directory.parent)
            for filename in filenames:
                abs_file_path = Path(folder_name).joinpath(filename).resolve()
                # avoid archiving the zip file itself
                if abs_file_path == Path(zip_filename).resolve():
                    continue
                rel_file_path = Path(relpath).joinpath(filename)
                zipObj.write(abs_file_path, rel_file_path)

    print(f"Zipping done: {zip_filename}")
    return zip_filename


def markdown_to_html(filepath):
    """
    Read a markdown file and return html transcript
    :param filepath: str or Path
    :return: str
        text in readme converted to html
    """
    html = markdown(open(filepath).read(), extensions=['fenced_code'])
    html = html.replace('\n', '')
    return html


def update_codemeta(
    codemeta_path=Path(ROOT_DIR).joinpath('codemeta.json').as_posix(),
    version=None,
    readme_path=None,
    modification_date=True,
    publication_date=True,
    download_url=None,
    release_notes=None,
    overwrite=True,
):
    """

    :param codemeta_path: Path or str
        path to the codemeta file to update
    :param version: str or None
    :param readme_path:
        Path to readme in markdown. An html transcript of its content will be used to update the `description` value.
    :param modification_date: bool
        If True, updates the modification date with today's
    :param publication_date: bool
        If True, updates the publication date with today's
    :param download_url: str
        Download URL for the software
    :param release_notes: str
        release notes. If None, does not update.
    :param overwrite: bool
        If True, overwrites the codemeta file
    """
    with open(codemeta_path) as file:
        metadata = json.load(file)

    if readme_path is not None:
        metadata['description'] = markdown_to_html(readme_path)

    if modification_date:
        metadata['dateModified'] = f'{date.today()}'

    if publication_date:
        metadata['datePublished'] = f'{date.today()}'

    if download_url is not None:
        metadata['downloadUrl'] = download_url
    if version is not None:
        metadata['version'] = f"v{version}"
        metadata['softwareVersion'] = f"v{version}"

    if release_notes is not None:
        metadata['releaseNotes'] = release_notes

    if overwrite:
        write_json(metadata, codemeta_path, overwrite=overwrite)
    return metadata


def spdx_licenses():
    """
    Get a list of all spdx licenses
    :return: list[str]
    """
    req = requests.get('https://raw.githubusercontent.com/spdx/license-list-data/master/json/licenses.json')
    try:
        req.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise requests.exceptions.HTTPError(f"Error while fetching spdx licenses: {e}") from e

    licenses = req.json()['licenses']
    return [lic['licenseId'] for lic in licenses]


def write_json(data, filename, overwrite=False):
    """
    Write data to a json file
    Parameters
    ----------
    data: dict
    filename: str or Path
    overwrite: bool (default: False)
        If True, overwrites the file if it exists
    """

    if Path(filename).exists() and not overwrite:
        raise FileExistsError(f"{filename} exists. Set overwrite=True")
    with open(filename, 'w') as file:
        json.dump(data, file, indent=4)



#!/usr/bin/env python

import concurrent.futures
import json
import os
import pprint
import sys
import warnings
from copy import deepcopy
from pathlib import Path
from urllib.parse import urlencode
from urllib.request import urlopen, urlretrieve

import requests
from bs4 import BeautifulSoup

from ...metadata.codemeta2zenodo import converter, parse_codemeta_and_write_zenodo_metadata_file
from ...metadata.zenodo import write_zenodo_metadata
from ...utils import get_codemeta_from_zipurl, write_json
from . import http_status

__all__ = [
    'zenodo_api_url',
    'zenodo_sandbox_api_url',
    'ZenodoAPI',
    'SimilarRecordError',  # noqa
    'Record',
    'query_zenodo_records',
    'get_zenodo_records',
    'query_record',
    'get_record',
    'get_supported_licenses',
    'search_records',
    'search_funders',
    'search_grants',
    'search_communities',
    'search_licenses',
    'is_live',
]

zenodo_api_url = "https://zenodo.org/api"
zenodo_sandbox_api_url = "https://sandbox.zenodo.org/api"

_default_size_query = 50


class ZenodoAPI:
    def __init__(self, access_token=None, sandbox=False):
        """
        Manages the communication with the (sandbox.)zenodo REST API through the Python request library.
        The client would allow to perform the following tasks within the (sandbox.)zenodo api environment:

          - Fetches a user's published entries,
          - Creates a new deposit,
          - Fetches any published record,
          - Creates a new version of an existing deposit,
          - Uploads files to a specific Zenodo deposit,
          - Erases a non-published deposit / new version draft,
          - Erases (old version) files from an deposit (when creating a new_version deposit and uploading
            new_version files),
          - Uploads information to the deposit (Zenodo compulsory deposit information),
          - Publishes an deposit
          - Finds all the published community entries
            * per title
            * per deposit_id
          - Finds all the records of a user (defined by the zenodo token)
          - Searches for similar records within all records associated to a user.

          Please note that every request.json() answer has been limited to 50 elements. You can set this value
          as follows (once ZenodoAPI has been initialised, for example):
          z = ZenodoApi(token)
          z.parameters.update({'size': INTEGER_NUMBER)

        :param access_token: str
            Personal access token to (sandbox.)zenodo.org/api
        :param sandbox: bool
            Communicates with either zenodo or sandbox.zenodo api
        """

        self.sandbox = sandbox
        self.api_url = zenodo_sandbox_api_url if sandbox else zenodo_api_url
        if access_token is None:
            warnings.warn("No access token provided, limited functionalities")
        self.access_token = access_token
        self.parameters = {'access_token': self.access_token}
        self.parameters.setdefault('size', _default_size_query)

    def _raise_token_status(self):
        """
        private method to check if a valid token has been provided, called in methods requiring a token
        :return:
        """
        if self.access_token is None or self.access_token == '':
            raise ValueError("No access token was provided. This method requires one.")

    def query_user_deposits(self):
        """
        Fetch the published entries of a user. Works to test connection to Zenodo too.

        GET method to {api_url}/deposit/depositions

        :return: request.get method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions"
        answer = requests.get(url, params=self.parameters)
        http_status.ZenodoHTTPStatus(answer.status_code, answer.json())
        return answer

    def query_deposit(self, deposit_id):
        """
        Fetches (recovers all the existing information, as well as links) of an existing Zenodo deposit.
        Parameters
        ----------
        deposit_id: str or int

        Returns
        -------

        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions/{deposit_id}"
        answer = requests.get(url, params=self.parameters)
        http_status.ZenodoHTTPStatus(answer.status_code, answer.json())
        return answer

    def create_new_deposit(self):
        """
        Create a new deposit in (sandbox.)zenodo

        POST method to {api_url}/deposit/depositions

        :return: request.put method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions"
        headers = {"Content-Type": "application/json"}
        req = requests.post(url, json={}, headers=headers, params=self.parameters)
        http_status.ZenodoHTTPStatus(req.status_code, req.json())
        return req

    def query_record(self, record_id):
        """
        Query (recovers all the existing information, as well as links) of a Zenodo record.

        GET method to {api_url}/deposit/depositions/{deposit_id}

        :param record_id: str
            deposit_id of the entry to fetch

        :return: request.get method
        """
        return query_record(record_id, sandbox=self.sandbox)

    def upload_file_deposit(self, deposit_id, name_file, path_file):
        """
        Upload a file to a Zenodo deposit. If first retrieve the deposit by a GET method to the
            {api_url}/deposit/depositions/{record_id}.

        PUT method to {bucket_url}/{filename}. The full api url is recovered when the deposit is firstly retrieved.

        :param deposit_id: str
            deposition_id of the Zenodo deposit
        :param name_file: str
            File name of the file when uploaded
        :param path_file: str
            Path to the file to be uploaded

        :return: request.put method
        """
        self._raise_token_status()
        # 1 - Retrieve and recover information of a record that is in process of being published
        fetch = requests.get(f"{self.api_url}/deposit/depositions/{deposit_id}", params=self.parameters)
        http_status.ZenodoHTTPStatus(fetch.status_code, fetch.json())

        # 2 - Upload the files
        bucket_url = fetch.json()['links']['bucket']  # full url is recovered from previous GET method
        url = f"{bucket_url}/{name_file}"

        with open(path_file, 'rb') as upload_file:
            upload = requests.put(url, data=upload_file, params=self.parameters)

        http_status.ZenodoHTTPStatus(upload.status_code, upload.json())
        return upload

    def set_deposit_metadata(self, deposit_id, json_metadata):
        """
        Set a deposit metadata. The metadata passed must be exhaustive, i.e. all the compulsory fields must be provided.

        PUT method to {api_url}/deposit/depositions/{deposit_id}. `data` MUST be included as json.dump(data)

        :param deposit_id: str
            deposition_id of the Zenodo deposit
        :param json_metadata: object
            json object containing the metadata (compulsory fields) that are enclosed when a new deposit is created.

        :return: request.put method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions/{deposit_id}"
        headers = {"Content-Type": "application/json"}

        # The metadata field is already created, just need to be updated.
        # Thus, the root 'metadata' key need to be kept, to indicate the field to be updated.
        data = {"metadata": json_metadata}
        req = requests.put(url, data=json.dumps(data), headers=headers, params=self.parameters)
        http_status.ZenodoHTTPStatus(req.status_code, req.json())
        return req

    def update_deposit_metadata(self, deposit_it, metadata):
        """
        Update the deposit metadata with only the one provided.

        Parameters
        ----------
        deposit_it: str or int
        metadata: dict
            The metadata to be updated.
        """
        # TODO: implement this method
        req = self.query_deposit(deposit_it)
        data = req.json()
        data['metadata'].update(metadata)
        req = self.set_deposit_metadata(deposit_it, data['metadata'])
        return req

    def erase_deposit(self, deposit_id):
        """
        Erase a deposit (that has not been published yet).
        Any new upload/version will be first saved as 'draft' and not published until confirmation (i.e, requests.post)

        DELETE method to {api_url}/deposit/depositions/{deposit_id}.

        :param deposit_id: str or int
            deposition_id of the Zenodo deposit to be erased

        :return: request.delete method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions/{deposit_id}"
        req = requests.delete(url, params=self.parameters)
        if req.status_code == 204:
            print("The deposit has been deleted")
            return req
        elif req.status_code == 410:  # Not raising an error in this case is OK
            warnings.warn("The deposit already was deleted")
        else:
            http_status.ZenodoHTTPStatus(req.status_code, req.json())
            return req

    def erase_file_deposit(self, deposit_id, file_id):
        """
        Erase a file from a deposit (that has not been published yet).
        This method is intended to be used for substitution of files (deletion) within a deposit by their correspondent
        new versions.

        DELETE method to {api_url}/deposit/depositions/{deposit_id}/files/{file_id}

        :param deposit_id: str
            deposition_id of the Zenodo deposit
        :param file_id: str
            ID of the files stored in Zenodo

        :return: requests.delete method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions/{deposit_id}/files/{file_id}"
        req = requests.delete(url, params=self.parameters)
        http_status.ZenodoHTTPStatus(req.status_code)
        return req

    def publish_deposit(self, deposit_id):
        """
        Publishes a deposit in (sandbox.)zenodo

        POST method to {api_url}/deposit/depositions/{deposit_id}/actions/publish

        :param deposit_id: str
            deposition_id of the Zenodo entry

        :return: requests.put method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions/{deposit_id}/actions/publish"
        req = requests.post(url, params=self.parameters)
        http_status.ZenodoHTTPStatus(req.status_code, req.json())
        return req

    def new_version_deposit(self, record_id):
        """
        Creates a new version of an existing record.

        POST method to {api_url}/deposit/depositions/{deposit_id}/actions/newversion

        :param record_id: str or int
        :return: requests.post method
        """
        self._raise_token_status()
        url = f"{self.api_url}/deposit/depositions/{record_id}/actions/newversion"
        parameters = {'access_token': self.access_token}
        req = requests.post(url, params=parameters)
        http_status.ZenodoHTTPStatus(req.status_code, req.json())
        return req

    def query_community_records(self, community_name='escape2020', **kwargs):
        """
        Query the records within a community.

        GET method, previous modification of the query arguments, to {api_url}/records

        :param community_name: str
            Community name.
        :param kwargs: dict
            Parameters for `query_zenodo_records`

        :return: `requests.models.Response`
        """
        # https://developers.zenodo.org/#list36
        parameters = deepcopy(self.parameters)
        parameters.update(kwargs)
        parameters['communities'] = str(community_name)
        return query_zenodo_records('', sandbox=self.sandbox, **parameters)

    @staticmethod
    def path_codemeta_file(root_dir):
        return Path(root_dir).joinpath('codemeta.json')

    @staticmethod
    def path_zenodo_file(root_dir):
        return Path(root_dir).joinpath('.zenodo.json')

    def upload_dir_content(self, directory, record_id=None, metadata=None, erase_previous_files=True, publish=True):
        """
        Package the project root directory as a zip archive and upload it to Zenodo.
        If a `record_id` is passed, a new version of that record is created. Otherwise, a new record is created.

        :param directory: Path or str
            path to the directory to upload
        :param record_id: str, int of None
            If a record_id is provided, a new version of the record will be created.
        :param metadata: dict or None
            dictionary of zenodo metadata
            if None, the metadata will be read from a `.zenodo.json` file or a `codemeta.json` file in `self.root_dir`
        :param erase_previous_files: bool
            In case of making a new version of an existing record (`record_id` not None), erase files from the previous
            version
        :param publish: bool
            If true, publish the record. Otherwise, the record is prepared but publication must be done manually. This
            is useful to check or discard the record before publication.
        """
        self._raise_token_status()
        # prepare new record version
        if record_id is not None:

            record = Record.from_id(record_id, sandbox=self.sandbox)
            record_id = record.last_version_id
            new_deposit = self.new_version_deposit(record_id)
            new_deposit_id = new_deposit.json()['links']['latest_draft'].rsplit('/')[-1]
            print(f" * Preparing a new version of record {record_id}")
            # TODO: log
            if erase_previous_files:
                old_files_ids = [file['id'] for file in new_deposit.json()['files']]
                for file_id in old_files_ids:
                    self.erase_file_deposit(new_deposit_id, file_id)
                    print(f"   - file {file_id} erased")
        else:

            new_deposit = self.create_new_deposit()
            new_deposit_id = new_deposit.json()['id']
            print(' * Preparing a new record')

        print(f" * New record id: {new_deposit_id}")

        # get metadata
        path_codemeta_file = self.path_codemeta_file(directory)
        path_zenodo_file = self.path_zenodo_file(directory)
        if metadata is not None:
            print(f" * Record metadata based on provided metadata: {metadata}")
        elif path_zenodo_file.exists():
            print(f"   - Record metadata based on zenodo file {path_zenodo_file}")
            with open(path_zenodo_file) as file:
                metadata = json.load(file)
        elif path_codemeta_file.exists():
            print(f"   - Record metadata based on codemeta file {path_codemeta_file}")
            with open(path_codemeta_file) as file:
                codemeta = json.load(file)
            metadata = converter(codemeta)
        else:
            raise FileNotFoundError(" ! No metadata file provided")

        # upload files
        dir_to_upload = Path(directory)
        for file in dir_to_upload.iterdir():
            self.upload_file_deposit(deposit_id=new_deposit_id, name_file=file.name, path_file=file)
            print(f" * {file.name} uploaded")

        # and update metadata
        self.set_deposit_metadata(new_deposit_id, json_metadata=metadata)
        print(" * Metadata updated successfully")

        # publish new record
        if publish:
            self.publish_deposit(new_deposit_id)
            if record_id:
                print(f" * New version of {record_id} published at {new_deposit_id} !")
            else:
                print(f" * Record {new_deposit_id} published")
            print(f" * The new doi should be 10.5281/{new_deposit_id}")

        print(f" * Check the upload at {self.api_url[:-4]}/deposit/{new_deposit_id} *")

        return new_deposit_id

    def check_upload_to_zenodo(self, directory):
        """
        `Tests` the different stages of the GitLab-Zenodo connection and that the status_code returned by every
        stage is the correct one.

        Checks:
         - The existence of a `.zenodo.json` file in the ROOT dir of the project
            - If not, it checks if it exists a `codemeta.json` file
               - If it exists it performs the codemeta2zenodo conversion
               - If not, it exits the program

         - The communication with Zenodo through its API to verify that:
            - You can fetch a user entries
            - You can create a new entry
            - The provided zenodo metadata can be digested, and not errors appear
            - Finally erases the test entry - because IT HAS NOT BEEN PUBLISHED !
        """
        self._raise_token_status()
        path_zenodo_file = self.path_zenodo_file(directory)
        path_codemeta_file = self.path_codemeta_file(directory)
        if not path_zenodo_file.exists():
            if not path_codemeta_file.exists():
                raise FileNotFoundError(f"No codemeta {path_codemeta_file} nor zenodo {path_zenodo_file} files.")

            print("\n * Creating a .zenodo.json file from your codemeta.json file...")

            parse_codemeta_and_write_zenodo_metadata_file(path_codemeta_file, path_zenodo_file)
        print(f"\n * Using {path_zenodo_file} file to simulate a new upload to Zenodo... \n")

        # 1 - Test connection
        print("1 --> Testing communication with Zenodo...")

        test_connection = self.query_user_deposits()

        http_status.ZenodoHTTPStatus(test_connection.status_code, test_connection.json())
        print("  * Test connection status OK !")

        # 2 - Test new entry
        print("2 --> Testing the creation of a dummy entry to (sandbox)Zenodo...")

        new_deposit = self.create_new_deposit()

        http_status.ZenodoHTTPStatus(new_deposit.status_code, new_deposit.json())
        print("  * Test new deposit status OK !")

        # 3 - Test upload metadata
        print("3 --> Testing the ingestion of the Zenodo metadata...")

        test_deposit_id = new_deposit.json()['id']
        with open(path_zenodo_file) as file:
            metadata_entry = json.load(file)
        updated_metadata = self.set_deposit_metadata(test_deposit_id, json_metadata=metadata_entry)

        try:
            http_status.ZenodoHTTPStatus(updated_metadata.status_code)
            print("  * Metadata deposit status OK !")
            pprint.pprint(metadata_entry)
        except http_status.HTTPStatusError:
            print("  ! ERROR while testing update of metadata\n", updated_metadata.json())
            print("  ! The deposit will be deleted")

        # 4 - Test delete entry
        print("4 --> Deleting the dummy entry...")
        delete_test_entry = self.erase_deposit(test_deposit_id)
        try:
            http_status.ZenodoHTTPStatus(delete_test_entry.status_code)
        except http_status.HTTPStatusError:
            print(f" !! ERROR erasing dummy test entry: {delete_test_entry.json()}")
            print(f"Please erase it manually at {self.api_url[:-4]}/deposit")
            sys.exit(-1)

        print("  * Delete test entry status OK !")

        print(
            "\n\tYAY ! Successful testing of the connection to Zenodo ! \n\n"
            "You should not face any trouble when uploading a project to Zenodo"
        )

    def get_user_records(self):
        """Finds all the records associated with a user (defined by the zenodo token)"""
        request = self.query_user_deposits()

        return [Record(hit) for hit in request.json() if hit['state'] == 'done']

    def find_similar_records(self, record):
        """
        Find similar records in the owner records.
        This check is not exhaustive and is based only on a limited number of parameters.

        :param record: `eossr.api.zenodo.Record`
        :return: list[Record]
            list of similar records
        """
        similar_records = []
        user_records = self.get_user_records()
        for user_rec in user_records:
            if user_rec.title == record.title:
                similar_records.append(user_rec)

            if 'related_identifiers' in user_rec.data['metadata'] and 'related_identifiers' in record.data['metadata']:

                relid1 = [r['identifier'] for r in user_rec.data['metadata']['related_identifiers']]
                relid2 = [r['identifier'] for r in record.data['metadata']['related_identifiers']]

                if set(relid1).intersection(relid2):
                    similar_records.append(user_rec)

        return similar_records

    def get_community_pending_requests(self, community, **params):
        """
        Get a list of records that have been requested to be added to a community.

        :param community: str
            Name of the community.
        :param params: dict
            Parameters for the request. Override the class parameters.
        :return: [Record]
        """
        self._raise_token_status()
        url = self.api_url + f'/records/?q=provisional_communities:{community.lower()}'
        parameters = deepcopy(self.parameters)
        parameters.update(params)
        req = requests.get(url, params=parameters)
        http_status.ZenodoHTTPStatus(req.status_code, json=req.json())
        req_json = req.json()
        records = [Record(rec) for rec in req_json['hits']['hits']]
        return records

    def accept_pending_request(self, community, record_id):
        """
        Accept a pending request into a community.
        The community must be owned by the token owner.

        :param community: str
            community name. The community must be owned by the token owner.
        :param record_id:
            str or int
        """
        self._raise_token_status()
        raise NotImplementedError("Sorry, we are working on it")
        # TODO: this should work based on https://github.com/zenodo/zenodo/issues/1436
        # run(['curl', '-i', '-X', 'POST', '-H',
        #      'Content-Type:application/json', '--data',
        #      f'{"action":"accept", "recid:{record_id}"}',
        #      f'"https://zenodo.org/communities/{community}/curate/"',
        #      ])

    def update_record_metadata(self, record_id, metadata):
        """
        Update a published record metadata

        :param record_id: int
        :param metadata: dict
        :return: `requests.response`
        """
        self._raise_token_status()
        req = requests.post(
            f"{self.api_url}/deposit/depositions/{record_id}/actions/edit?access_token={self.access_token}"
        )
        if req.status_code == 403:
            # In this case it is fine to continue editing the record metadata
            warnings.warn("The record was already open for edition")
        else:
            http_status.ZenodoHTTPStatus(req.status_code, req.json())

        record = get_record(record_id, sandbox=self.sandbox)
        record_metadata = record.data['metadata']
        record_metadata['upload_type'] = record_metadata['resource_type']['type']
        record_metadata.pop('access_right_category')
        record_metadata.pop('relations')
        record_metadata.pop('related_identifiers')
        record_metadata.pop('resource_type')
        record_metadata.update(metadata)
        self.set_deposit_metadata(record_id, json_metadata=record_metadata)
        req = self.publish_deposit(record_id)
        return req


class SimilarRecordError(Exception):
    pass


class Record:
    """
    Basic class object to handle Zenodo records
    """

    def __init__(self, data: dict):
        for k in ['id', 'metadata']:
            if k not in data.keys():
                raise ValueError(f"key {k} not present in data")
        # list of keys mandatory to create a Zenodo entry.
        # Other keys are either optional, or can be hidden in case of Closed Access entries.
        for meta_key in ['title', 'doi']:
            if meta_key not in data['metadata'].keys():
                raise ValueError(f"Mandatory key {meta_key} not in data['metadata']")
        self.data = data

    def __str__(self):
        return f"Record #{self.id} : {self.title}"

    def __repr__(self):
        return f"Record({self.id})"

    def _write_zenodo_deposit(self, filename='.zenodo.json', overwrite=False, validate=True):
        """
        Write the zenodo metadata to a `.zenodo.json` file, so it can be used to create a new deposit.
        The created is not guaranteed to be valid, but it is a good starting point.

        :param filename: str
            path to the file to write
        :param overwrite: bool
            True to overwrite existing file
        :param validate: bool
            True to validate the metadata before writing the file
        """
        # Transform metadata from record to deposit first
        metadata = deepcopy(self.data['metadata'])
        metadata['upload_type'] = metadata['resource_type']['type']
        metadata.pop('resource_type')
        metadata.pop('access_right_category')
        if 'relations' in metadata:
            metadata.pop('relations')
        if 'communities' in metadata:
            metadata['communities'] = [{'identifier': c['id']} for c in metadata['communities']]
        metadata['related_identifiers'] = [
            rid
            for rid in metadata['related_identifiers']
            if not (rid['relation'] == 'isVersionOf' and 'zenodo' in rid['identifier'])
        ]
        if 'zenodo' in metadata['doi']:
            metadata.pop('doi')
        metadata['license'] = metadata['license']['id']

        write_zenodo_metadata(metadata, filename=filename, overwrite=overwrite, validate=validate)

    def write_metadata(self, filename, overwrite=False):
        """
        Write the metadata to a json file.

        Parameters
        ----------
        filename: str
        overwrite: bool
            True to overwrite existing file
        """
        write_json(self.data['metadata'], filename=filename, overwrite=overwrite)

    @property
    def id(self):
        return self.data['id']

    @property
    def title(self):
        return self.data['metadata']['title']

    @property
    def metadata(self):
        return self.data['metadata']

    @property
    def filelist(self):
        """
        Return the list of files in the record

        :return: [str]
        """
        return [f['links']['self'] for f in self.data['files']]

    @property
    def last_version_id(self):
        """
        Return the ID of the last version of this record.
        If there is no other version, returns self.id

        :return: int
        """
        if 'relations' not in self.data['metadata']:
            return self.id
        else:
            return self.data['metadata']['relations']['version'][0]['last_child']['pid_value']

    def get_last_version(self):
        """
        Return the last version of the record.
        If there is only one version, or if this is already the last version, return itself.

        :return: `eossr.api.zenodo.Record`
        """
        if 'relations' not in self.data['metadata'] or self.data['metadata']['relations']['version'][0]['is_last']:
            return self
        record_id = self.last_version_id
        url = Path(self.data['links']['self']).parent.joinpath(str(record_id)).as_posix()
        return Record(requests.get(url).json())

    @property
    def from_sandbox(self):
        """
        Is the record from sandbox?
        :return: bool
        """
        if 'sandbox' in self.data['links']['self']:
            return True
        else:
            return False

    def get_associated_versions(self, size=_default_size_query, **kwargs):
        """
        Returns a dictionary of all the versions of the current record

        :param size: int
            Number of results to return. Default = 50 (`_default_size_query`)
        :param kwargs: Zenodo query arguments.
            For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36

        :return: dict
            dictionary of `{record_id: record_version}`
        """
        conceptrecid = self.data['conceptrecid']
        params = {'all_versions': True, **kwargs}
        params.setdefault('size', size)

        versions = {}
        for record in get_zenodo_records(f'conceptrecid:{conceptrecid}', sandbox=self.from_sandbox, **params):
            if 'version' in record.metadata:
                versions[record.id] = record.metadata['version']
            else:
                versions[record.id] = None
        return versions

    def print_info(self, linebreak='\n', file=sys.stdout):
        """
        Print general information about the record to a stream, or to sys.stdout by default.

        :param linebreak: string
            line break character. default: '\n'
        :param file: a file-like object (stream); defaults to the current sys.stdout.
        :return:
        """
        metadata = self.data['metadata']
        descrp = f"=== Record #{self.id} === {linebreak}"
        descrp += f"Title: {self.title} {linebreak}"
        version = self.metadata['version'] if 'version' in self.metadata else 'Unknown'
        descrp += f"Version: {version} {linebreak}"
        descrp += f"DOI: {self.data['doi']} {linebreak}"
        if 'links' in self.data.keys() and 'html' in self.data['links'].keys():
            descrp += f"URL: {self.data['links']['html']} {linebreak}"
        if 'description' in metadata:
            descrp += BeautifulSoup(metadata['description'], features='html.parser').get_text()
        descrp += f"{linebreak}"
        print(descrp, file=file)

    @classmethod
    def from_id(cls, record_id, sandbox=False):
        """
        Retrieve a record from its record id.

        :param record_id: int
        :param sandbox: bool
            True to use Zenodo's sandbox

        :return: `eossr.api.zenodo.Record`
        """
        record = get_record(record_id=record_id, sandbox=sandbox)
        return record

    def get_codemeta(self, **zipurl_kwargs):
        """
        Get codemeta metadata from the record (can also be in a zip archive).
        Raises an error if no `codemeta.json` file is found.

        :param zipurl_kwargs: dict
            kwargs for `eossr.utils.ZipUrl`

        :return: dict
            codemeta metadata
        """
        if 'files' not in self.data:
            raise FileNotFoundError(f'The record {self.id} does not contain any file')

        codemeta_paths = [s for s in self.filelist if Path(s).name == 'codemeta.json']
        ziparchives = [s for s in self.filelist if s.endswith('.zip')]
        if len(codemeta_paths) >= 1:
            # if there are more than one codemeta file in the repository, we consider the one in the root directory,
            # hence the one with the shortest path
            chosen_codemeta = min(codemeta_paths, key=len)
            return json.loads(urlopen(chosen_codemeta).read())
        elif len(ziparchives) > 0:
            for zipurl in ziparchives:
                try:
                    return get_codemeta_from_zipurl(zipurl, **zipurl_kwargs)
                except FileNotFoundError:
                    pass
            raise FileNotFoundError(f"No `codemeta.json` file found in record {self.id}")
        else:
            raise FileNotFoundError(f"No `codemeta.json` file found in record {self.id}")

    @property
    def doi(self):
        if 'doi' not in self.data:
            raise KeyError(f"Record {self.id} does not have a doi")
        return self.data['doi']

    def get_mybinder_url(self):
        """
        Returns a URL to a mybinder instance of that record

        :return: str
        """
        binder_zenodo_url = 'https://mybinder.org/v2/zenodo/'
        doi = self.doi
        return binder_zenodo_url + doi

    def download(self, directory='.', max_workers=None):
        """
        Download the record to a directory.

        Parameters
        ----------
        directory: str or Path
            Directory where to download the record content
        max_workers: int or None
            Number of workers to use for the download. If None, use all available workers.
        """
        # Create the directory if it doesn't exist
        os.makedirs(directory, exist_ok=True)
        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            # Start the load operations and mark each future with its URL
            future_to_url = {
                executor.submit(urlretrieve, url, f'{directory}/{os.path.basename(url)}'): url for url in self.filelist
            }
            for future in concurrent.futures.as_completed(future_to_url):
                url = future_to_url[future]
                try:
                    future.result()
                    print(f'{url} : Download complete')
                except Exception as exc:
                    print(f'{url} generated an exception: {exc}')


def query_zenodo_records(search='', sandbox=False, **kwargs):
    """
    Query Zenodo for records whose names or descriptions include the provided string `search`.
    Function rewritten from pyzenodo3 (https://github.com/space-physics/pyzenodo3)

    :param search: string
        A string to refine the search in Zenodo. The default will search for all records.
    :param sandbox: bool
        Indicates the use of sandbox zenodo or not.
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 50
            - all_versions: int
                Show (1) or hide (0) all versions of records
            - type: string or list[string]
                Records of the specified type (Publication, Poster, Presentation, Software, ...)
                A logical OR is applied in case of a list
            - keywords: string or list[string]
                Records with the specified keywords
                 A logical OR is applied in case of a list
            - communities: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
            - file_type: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list

    :return:
    `requests.models.Response`
    """
    search = search.replace("/", " ")  # zenodo can't handle '/' in search query

    params = {'q': search, **kwargs}

    params.setdefault('size', str(_default_size_query))

    def lowercase(param):
        if isinstance(param, str):
            param = param.lower()
        if isinstance(param, list):
            param = [char.lower() for char in param]
        return param

    for param_name in ['communities', 'type', 'file_type']:
        if param_name in kwargs:
            params[param_name] = lowercase(kwargs[param_name])

    api_url = zenodo_sandbox_api_url if sandbox else zenodo_api_url
    url = api_url + "/records?" + urlencode(params, doseq=True)
    req = requests.get(url)
    http_status.HTTPStatusError(req.status_code, req.json())
    return req


def get_zenodo_records(search='', sandbox=False, **kwargs):
    """
    Search Zenodo for records whose names or descriptions include the provided string `search`.
    Function rewritten from pyzenodo3 (https://github.com/space-physics/pyzenodo3)

    :param search: string
        A string to refine the search in Zenodo. The default will search for all records.
    :param sandbox: bool
        Indicates the use of sandbox zenodo or not.
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 100
            - all_versions: int
                Show (1) or hide (0) all versions of records
            - type: string or list[string]
                Records of the specified type (Publication, Poster, Presentation, Software, ...)
                A logical OR is applied in case of a list
            - keywords: string or list[string]
                Records with the specified keywords
                 A logical OR is applied in case of a list
            - communities: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
            - file_type: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list

    :return: [Record]
        list of records
    """
    answer = query_zenodo_records(search=search, sandbox=sandbox, **kwargs)
    hits = answer.json()["hits"]["hits"]
    if not hits:
        raise LookupError(f"No records found for search {search}")
    else:
        return [Record(hit) for hit in hits]


def query_record(record_id, sandbox=False):
    """
    Send a request for a record to Zenodo (or its sandbox if `sandbox=True`).

    :param record_id: int
        record ID
    :param sandbox: boolean
        whether to request in the sandbox or not
    :return:
    `requests.models.Response`
    """
    api_url = zenodo_sandbox_api_url if sandbox else zenodo_api_url
    url = f"{api_url}/records/{record_id}"
    return requests.get(url)


def get_record(record_id, sandbox=False):
    """
    Get a record from its id

    :param record_id: int or str
        Zenodo record id number.
    :param sandbox: bool
        Indicates the use of sandbox zenodo or not.

    :return: Record
    """
    answer = query_record(record_id, sandbox=sandbox)
    answer_json = answer.json()
    http_status.ZenodoHTTPStatus(answer.status_code, answer_json)
    return Record(answer_json)


def get_supported_licenses(size=1000):
    """
    Recovers the list of Zenodo supported license IDs and names.
    Makes a request.get() call to Zenodo.

    :return: list
        license id plus license name Zenodo supported licenses list
    """
    licenses = search_licenses(size=size)
    return [license['metadata']['id'] for license in licenses]


def _query(field, search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param field: str
        where to search: 'records', 'funders', 'grants', 'communities', 'licenses'
    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 100
            - all_versions: int
                Show (1) or hide (0) all versions of records
            - type: string or list[string]
                Records of the specified type (Publication, Poster, Presentation, Software, ...)
                A logical OR is applied in case of a list
            - keywords: string or list[string]
                Records with the specified keywords
                 A logical OR is applied in case of a list
            - communities: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
            - file_type: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
    :return: `requests.response`
    """

    def lowercase(param):
        if isinstance(param, str):
            param = param.lower()
        if isinstance(param, list):
            param = [char.lower() for char in param]
        return param

    search = search.replace("/", " ")  # zenodo can't handle '/' in search query

    params = {'q': search, **kwargs}

    params.setdefault('size', 100)

    for param_name in ['communities', 'type', 'file_type']:
        if param_name in kwargs:
            params[param_name] = lowercase(kwargs[param_name])

    api_url = zenodo_api_url if not sandbox else zenodo_sandbox_api_url
    url = api_url + f"/{field}?" + urlencode(params, doseq=True)

    response = requests.get(url)
    return response


def _search(field, search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param field: str
        where to search: 'records', 'funders', 'grants', 'communities', 'licenses'
    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 100
            - all_versions: int
                Show (1) or hide (0) all versions of records
            - type: string or list[string]
                Records of the specified type (Publication, Poster, Presentation, Software, ...)
                A logical OR is applied in case of a list
            - keywords: string or list[string]
                Records with the specified keywords
                 A logical OR is applied in case of a list
            - communities: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
            - file_type: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
    :return: [dict]
    """

    query = _query(field, search=search, sandbox=sandbox, **kwargs)
    http_status.ZenodoHTTPStatus(query.status_code, query.json())

    hits = [hit for hit in query.json()["hits"]["hits"]]
    return hits


def search_records(search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 100
            - all_versions: int
                Show (1) or hide (0) all versions of records
            - type: string or list[string]
                Records of the specified type (Publication, Poster, Presentation, Software, ...)
                A logical OR is applied in case of a list
            - keywords: string or list[string]
                Records with the specified keywords
                 A logical OR is applied in case of a list
            - communities: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
            - file_type: string or list[string]
                Records from the specified keywords
                A logical OR is applied in case of a list
    :return: [dict]
    """
    hits = _search('records', search=search, sandbox=sandbox, **kwargs)
    return hits


def search_funders(search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 5
    :return: [dict]
    """
    kwargs.setdefault('size', 5)
    hits = _search('funders', search=search, sandbox=sandbox, **kwargs)
    return hits


def search_grants(search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 5
    :return: [dict]
    """
    kwargs.setdefault('size', 5)
    hits = _search('grants', search=search, sandbox=sandbox, **kwargs)
    return hits


def search_communities(search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 5
    :return: [dict]
    """
    kwargs.setdefault('size', 5)
    hits = _search('communities', search=search, sandbox=sandbox, **kwargs)
    return hits


def search_licenses(search='', sandbox=False, **kwargs):
    """
    https://help.zenodo.org/guides/search/

    :param search: str
    :param sandbox: boolean
        True to search in the sandbox
    :param kwargs: Zenodo query arguments.
        For an exhaustive list, see the query arguments at https://developers.zenodo.org/#list36
        Common arguments are:
            - size: int
                Number of results to return
                Default = 5
    :return: [dict]
    """
    kwargs.setdefault('size', 5)
    hits = _search('licenses', search=search, sandbox=sandbox, **kwargs)
    return hits


def is_live(sandbox=False):
    """
    Check if Zenodo website is live
    :param sandbox: bool
        True to test sandbox instead
    :return: bool
        True if live
    """
    url = zenodo_sandbox_api_url if sandbox else zenodo_api_url
    req = requests.get(url)
    return req.status_code == 200


def query_deposit(deposit_id, access_token, sandbox=False):
    """
    Query a deposit

    Parameters
    ----------
    deposit_id: str or int

    Returns
    -------

    """
    url = zenodo_api_url + 'deposit/depositions/{}'.format(deposit_id)
    response = requests.get(url, params={'access_token': access_token})
    return response



#!/usr/bin/env python
import argparse
import json
from pathlib import Path

from eossr.metadata import zenodo


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(
        description="Validate a Zenodo metadata file (.zenoodo.json). "
        "Raises warnings for recommended changes "
        "and errors for unvalid entries"
    )

    parser.add_argument('filename', type=Path, help='Path to .zenodo.json')

    return parser


def main():

    parser = build_argparser()
    args = parser.parse_args()

    with open(args.filename, 'r') as f:
        metadata = json.load(f)

    zenodo.validate_zenodo_metadata_deposit(metadata)

    print("Valid Zenodo metadata 🚀")


if __name__ == '__main__':
    main()



import tempfile
from os import path
from pathlib import Path

import pytest

from eossr.metadata import codemeta

SAMPLES_DIR = Path(path.join(path.dirname(path.realpath(__file__)), "samples"))


def test_samples_dir():
    assert SAMPLES_DIR.exists()


def test_codemeta():
    meta = codemeta.schema()
    assert meta['@context']['codemeta'] == "https://codemeta.github.io/terms/"


def test_codemeta_valid():
    valid_codemeta = SAMPLES_DIR.joinpath("codemeta_valid.json")
    codemeta_handler = codemeta.Codemeta.from_file(valid_codemeta)
    codemeta_handler.validate()


@pytest.mark.xfail(raises=codemeta.CodemetaRequiredError)
def test_codemeta_not_valid():
    not_valid_codemeta = SAMPLES_DIR.joinpath("codemeta_not_valid.json")
    codemeta_handler = codemeta.Codemeta.from_file(not_valid_codemeta)
    codemeta_handler.validate()


def test_read_codemeta_crosswalk():
    from numpy import nan

    cw = codemeta.codemeta_crosswalk()
    assert set(cw['OSSR Requirement Level'].values) == {'required', 'recommended', nan}


def test_codemeta_write():
    metadata = {'name': 'Rick', 'version': 'v10'}
    codemeta_handler = codemeta.Codemeta(metadata)
    with tempfile.TemporaryDirectory() as tmpdirname:
        filename = Path(tmpdirname, 'codemeta.json')
        codemeta_handler.write(filename)
        new_codemeta = codemeta.Codemeta.from_file(filename)
        assert new_codemeta.metadata == metadata



# Various tests taken/based from
# https://github.com/cta-observatory/cta-lstchain/blob/master/lstchain/scripts/tests/test_lstchain_scripts.py

import os
import shutil
import subprocess
from os.path import dirname, join, realpath
from pathlib import Path

import pkg_resources
import pytest

from eossr.scripts import eossr_upload_repository

ROOT_DIR = dirname(realpath("codemeta.json"))


def find_entry_points(package_name):
    """from: https://stackoverflow.com/a/47383763/3838691"""
    entrypoints = [
        ep.name for ep in pkg_resources.iter_entry_points("console_scripts") if ep.module_name.startswith(package_name)
    ]
    return entrypoints


ALL_SCRIPTS = find_entry_points("eossr")


def run_script(*args):
    result = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, encoding='utf-8')

    if result.returncode != 0:
        raise ValueError(
            f"Running {args[0]} failed with return code {result.returncode}" f", output: \n {result.stdout}"
        )


def test_codemeta2zenodo():
    existing_zenodo_file = Path(ROOT_DIR).joinpath('.zenodo.json')
    if existing_zenodo_file.exists():
        existing_zenodo_file.unlink()

    run_script("eossr-codemeta2zenodo", "-i", join(ROOT_DIR, "codemeta.json"))
    existing_zenodo_file.unlink()


@pytest.mark.parametrize("script", ALL_SCRIPTS)
def test_help_all_scripts(script):
    """Test for all scripts if at least the help works"""
    run_script(script, "--help")


@pytest.mark.skipif(os.getenv('SANDBOX_ZENODO_TOKEN') is None, reason="SANDBOX_ZENODO_TOKEN not defined")
def test_eossr_upload_repository(tmpdir):
    path_test_filename = Path(tmpdir).joinpath('test.txt')
    Path(path_test_filename).write_text('Hello World')
    shutil.copy(Path(ROOT_DIR).joinpath('codemeta.json'), tmpdir)
    eossr_upload_repository.upload(
        zenodo_token=os.getenv('SANDBOX_ZENODO_TOKEN'),
        sandbox_flag=True,
        upload_directory=tmpdir,
        force_new_record=True,
        publish=False,
    )


def test_eossr_metadata_validator_valid():
    from eossr.metadata.tests.test_codemeta import SAMPLES_DIR

    path_test_codemeta = SAMPLES_DIR.joinpath('codemeta_valid.json')
    run_script('eossr-metadata-validator', path_test_codemeta)


@pytest.mark.xfail()
def test_eossr_metadata_validator_not_valid():
    from eossr.metadata.tests.test_codemeta import SAMPLES_DIR

    path_test_codemeta = SAMPLES_DIR.joinpath('codemeta_not_valid.json')
    run_script('eossr-metadata-validator', path_test_codemeta)


@pytest.mark.skipif(os.getenv('SANDBOX_ZENODO_TOKEN') is None, reason="SANDBOX_ZENODO_TOKEN not defined")
def test_user_entries_cleanup():
    from eossr.scripts.zenodo_user_entries_cleanup import zenodo_cleanup

    zenodo_cleanup(token=os.getenv('SANDBOX_ZENODO_TOKEN'), sandbox=True)



#!/usr/bin/env python
"""
Simple code to delete all user entries that have not been published
"""
import argparse
import os
import warnings

from eossr.api.zenodo import ZenodoAPI
from eossr.api.zenodo.http_status import HTTPStatusError


def zenodo_cleanup(token, sandbox=True):
    """
    Delete user unpublished entries

    :param token: str
        Zenodo access token
    :param sandbox: bool
        True to use sandbox
    """
    zen = ZenodoAPI(token, sandbox=sandbox)
    use = zen.query_user_deposits()

    for rec in use.json():
        if not rec['submitted']:
            print(f"Record {rec['id']} ... ")
            try:
                zen.erase_deposit(rec['id'])
            except HTTPStatusError as e:
                warnings.warn(f"Error deleting record {rec['id']}:\n{e}")
                continue


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(description="Delete user unpublished entries. Working on sandbox by default.")

    parser.add_argument(
        '--token',
        '-t',
        type=str,
        default=os.getenv('SANDBOX_ZENODO_TOKEN'),
        help='Access token',
    )
    parser.add_argument('--not-sandbox', action='store_false', help="To work on the actual Zenodo instead of sandbox")
    return parser


def main():

    parser = build_argparser()
    args = parser.parse_args()
    zenodo_cleanup(args.token, args.not_sandbox)


if __name__ == '__main__':
    main()



from semver import VersionInfo


def valid_semver(version):
    """Return True if the version is valid according to semver.org
    Strips leading 'v' from the version string.
    """
    return VersionInfo.isvalid(version.lstrip("v"))



#!/usr/bin/env python

from pathlib import Path

from setuptools import find_packages, setup

entry_points = {
    'console_scripts': [
        'eossr-zip-repository = eossr.scripts.zip_repository:main',
        'eossr-codemeta2zenodo = eossr.scripts.eossr_codemeta2zenodo:main',
        'eossr-upload-repository = eossr.scripts.eossr_upload_repository:main',
        'eossr-check-connection-zenodo = eossr.scripts.check_connection_zenodo:main',
        'eossr-metadata-validator = eossr.scripts.eossr_metadata_validator:main',
        'eossr-zenodo-metadata-validator = eossr.scripts.eossr_zenodo_validator:main',
    ]
}


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(
    name='eossr',
    description="ESCAPE OSSR library",
    long_description=long_description,
    long_description_content_type='text/markdown',
    setup_requires="setuptools_scm",
    install_requires=[
        "requests>=2.25.0,<3.0",
        "beautifulsoup4>=4.10.0,<5.0",
        "markdown>=3.3.6,<4.0",
        "pandas",
        "remotezip==0.9.3",
        "semver>=2,<3",
        "jsonschema",
        "jsonref",
    ],
    extras_require={'tests': ['pytest', 'pytest-cov'], 'extras': ['pre-commit']},
    packages=find_packages(exclude="eossr._dev_version"),
    scripts=[],
    tests_require=['pytest'],
    author='Thomas Vuillaume, Enrique Garcia',
    author_email='vuillaume@lapp.in2p3.fr',
    url='https://gitlab.in2p3.fr/escape2020/wp3/eossr',
    license='MIT',
    entry_points=entry_points,
    package_data={
        'eossr': [
            'metadata/schema/.zenodo.json',
            'metadata/schema/codemeta.json',
            'metadata/schema/escape_codemeta_crosswalk.csv',
        ]
    },
    use_scm_version={
        "write_to": Path(__file__).parent.joinpath("eossr/_version.py"),
        "write_to_template": "__version__ = '{version}'",
    },
)



import json
from warnings import warn

import numpy as np
import pandas as pd
import pkg_resources

from ..utils import write_json
from . import valid_semver

__all__ = [
    'schema',
    'codemeta_crosswalk',
    'Codemeta',
]


def schema():
    return json.load(pkg_resources.resource_stream(__name__, 'schema/codemeta.json'))


def codemeta_crosswalk():
    return pd.read_csv(
        pkg_resources.resource_stream(__name__, 'schema/escape_codemeta_crosswalk.csv'), comment='#', delimiter=';'
    )


class CodemetaRequiredError(KeyError):
    counter = 0

    def __init__(self, message):
        CodemetaRequiredError.counter += 1


class CodemetaRecommendedWarning(Warning):
    counter = 0

    def __init__(self, message):
        CodemetaRecommendedWarning.counter += 1


class Codemeta:
    def __init__(self, metadata: dict):
        self.metadata = metadata
        self._crosswalk_table = None

    @classmethod
    def from_file(cls, codemeta_filename):
        """Load `codemeta_filename` into the validator"""
        with open(codemeta_filename) as infile:
            controller = cls(json.load(infile))
        return controller

    @property
    def schema(self):
        return schema()

    @property
    def crosswalk_table(self):
        if self._crosswalk_table is None:
            self._crosswalk_table = codemeta_crosswalk()
        return self._crosswalk_table

    def missing_keys(self, level='required'):
        """
        Return the list of keys that are required but not present in the metadata
        level: str
            'required' or 'recommended'
        """
        required_mask = self.crosswalk_table['OSSR Requirement Level'] == level
        keys = np.array(list(self.metadata.keys()))
        required_keys = self.crosswalk_table['Property'][required_mask].values
        missing_keys = required_keys[np.in1d(required_keys, keys, invert=True)]
        return missing_keys

    def validate(self):
        """Validate the metadata against the CodeMeta schema
        Raises errors for required keys and warnings for recommended keys
        """
        if self.missing_keys('required').size > 0:
            raise CodemetaRequiredError(
                f"Missing {self.missing_keys('required').size} required keys: " f"{self.missing_keys('required')}"
            )
        else:
            if self.missing_keys('recommended').size > 0:
                warn(
                    f"Missing {self.missing_keys('recommended').size} recommended keys: "
                    f"{self.missing_keys('recommended')}",
                    CodemetaRecommendedWarning,
                )
            if "version" in self.metadata and not valid_semver(self.metadata["version"]):
                warn(f"Version {self.metadata['version']} does not follow the recommended format from semver.org.")
            if "softwareVersion" in self.metadata and not valid_semver(self.metadata["softwareVersion"]):
                warn(
                    f"Version {self.metadata['softwareVersion']} does not follow the recommended format from "
                    f"semver.org."
                )
            return False
        print("CodeMeta is valid")
        return True

    def write(self, path='codemeta.json', overwrite=False):
        """Write the CodeMeta file to `path`"""
        write_json(self.metadata, path, overwrite=overwrite)



import os
import tempfile
from pathlib import Path
from zipfile import ZipFile

import pytest
import urllib3

from eossr import utils as eutils

_testurl = 'https://zenodo.org/record/5592584/files/eossr.zip'
_testurl_buffer = 'https://zenodo.org/record/5524913/files/eossr-v0.2.zip'


def test_ZipUrl():
    zipurl = eutils.ZipUrl(_testurl)
    codemeta_paths = zipurl.find_files('codemeta.json')

    assert 'eossr/codemeta.json' in codemeta_paths
    assert 'eossr/eossr/metadata/schema/codemeta.json' in codemeta_paths
    zipurl.extract('eossr/codemeta.json')
    assert Path('eossr/codemeta.json').exists()


@pytest.mark.xfail(urllib3.exceptions.ProtocolError)
def test_ZipUrl_buffer_fail():
    eutils.ZipUrl(_testurl_buffer, initial_buffer_size=64 * 1024)


def test_ZipUrl_buffer_success():
    zipurl = eutils.ZipUrl(_testurl_buffer)
    zipurl.extract('eossr-v0.2/codemeta.json')
    assert Path('eossr-v0.2/codemeta.json').exists()


def test_get_codemeta_from_zipurl():
    codemeta = eutils.get_codemeta_from_zipurl(_testurl, timeout=10, initial_buffer_size=100)
    assert codemeta['name'] == 'eossr'


def test_zip_repository():
    from eossr import ROOT_DIR
    from eossr.utils import zip_repository

    zip_file = zip_repository(ROOT_DIR, 'test_zipping_v0.1.zip')
    assert Path(zip_file).exists()
    with ZipFile(zip_file) as zo:
        for file in zo.namelist():
            assert '.git/' not in file
    Path(zip_file).unlink()

    with tempfile.TemporaryDirectory() as tmpdir:
        subtmpdir = tempfile.mkdtemp(dir=tmpdir)
        _, tmpfile = tempfile.mkstemp(dir=subtmpdir)
        Path(tmpfile).write_text("Hello World")
        zip_filename = Path(subtmpdir).joinpath('test_zip_repo.zip').as_posix()
        zip_repository(tmpdir, zip_filename=zip_filename)
        with ZipFile(zip_filename) as zo:
            assert 'test_zip_repo.zip' not in [Path(f).name for f in zo.namelist()]
            assert (
                os.path.join(
                    os.path.join(os.path.basename(tmpdir), os.path.basename(subtmpdir)), os.path.basename(tmpfile)
                )
                in zo.namelist()
            )


def test_markdown_to_html():
    _, tmpfile = tempfile.mkstemp()
    Path(tmpfile).write_text("# Hello Markdown")
    html = eutils.markdown_to_html(tmpfile)
    assert html == '<h1>Hello Markdown</h1>'


def test_update_codemeta():
    # do not update main codemeta.json in unit test
    eutils.update_codemeta(overwrite=False)


def test_spdx():
    assert "MIT" in eutils.spdx_licenses()
    assert "dummy" not in eutils.spdx_licenses()



#!/usr/bin/env python

import argparse
import sys
from pathlib import Path

from eossr.metadata.codemeta2zenodo import parse_codemeta_and_write_zenodo_metadata_file


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(
        description="Converts a metadata descriptive files from the the CodeMeta to the Zenodo schema. "
        "Creates a .zenodo.json file from a codemeta.json file."
    )

    parser.add_argument(
        '--input_codemeta_file',
        '-i',
        type=str,
        dest='codemeta_file',
        help='Path to a codemeta.json file',
        required=True,
    )
    parser.add_argument('--overwrite', action='store_true', help='Use to overwrite an existing `.zenodo.json` file')

    return parser


def main():
    """
    Check files exist and run codemeta2zenodo
    """

    parser = build_argparser()
    args = parser.parse_args()

    codemeta_file = Path(args.codemeta_file)

    # Check if file exists and it is named as it should
    if not codemeta_file.exists():
        raise FileNotFoundError(f"Codemeta file {codemeta_file} not found")
        sys.exit(-1)

    if not codemeta_file.name.startswith('codemeta') or not codemeta_file.name.endswith('.json'):
        raise ValueError(
            f"\n\t{codemeta_file.name} either does not starts with the `codemeta` prefix or "
            f"either it does not finishes with a `.json` suffix. Exiting"
        )
        sys.exit(-1)

    directory_codemeta = codemeta_file.parent.absolute()
    zenodo_metadata_file = directory_codemeta / '.zenodo.json'

    # Check overwrite zenodo file if exists
    if zenodo_metadata_file.exists() and not args.overwrite:
        raise FileExistsError(f"File {zenodo_metadata_file} exists. Use --overwrite to overwrite file")
    else:
        # Parse the codemeta.json file and create the .zenodo.json file
        parse_codemeta_and_write_zenodo_metadata_file(
            codemeta_file, outdir=directory_codemeta, overwrite=args.overwrite
        )
    print("\nConversion codemeta2zenodo done.\n")


if __name__ == "__main__":
    main()



from pathlib import Path

import eossr
from eossr.metadata import codemeta


def test_eossr_codemeta():
    eossr_codemeta_file = Path(__file__).parent.joinpath('../../codemeta.json').resolve()
    codemeta_handler = codemeta.Codemeta.from_file(eossr_codemeta_file)
    codemeta_handler.validate()


def test_version():
    assert eossr.__version__ != "0.0.0"



import argparse
import os
from pathlib import Path

import requests

from eossr import ROOT_DIR
from eossr import __version__ as eossr_version
from eossr.utils import update_codemeta


def build_argparser():
    """
    Construct main argument parser for the ``codemet2zenodo`` script

    :return:
    argparser: `argparse.ArgumentParser`
    """
    parser = argparse.ArgumentParser(description="Update Codemeta")

    parser.add_argument(
        '--codemeta_path',
        '-c',
        type=Path,
        dest='codemeta_path',
        help='Path to codemeta.json',
        default=Path(ROOT_DIR).joinpath('codemeta.json'),
        required=False,
    )

    parser.add_argument(
        '--no-release',
        action='store_true',
        help="Use when making a release. "
        "Do not update the publication date, the zip archive URL and remove release notes.",
    )

    return parser


def get_gitlab_releases(api_url, project_id, token):
    """
    Get the releases from Gitlab

    Parameters
    ----------
    api_url: str
    project_id: int or str
    token: str

    Returns
    -------
    releases: list[dict]
    """
    releases_url = f"{api_url}/projects/{project_id}/releases"
    req = requests.get(releases_url, params={"PRIVATE-TOKEN": token})
    req.raise_for_status()
    return req.json()


if __name__ == '__main__':
    parser = build_argparser()
    args = parser.parse_args()

    project_id = os.getenv('CI_PROJECT_ID')
    api_url = os.getenv('CI_API_V4_URL')
    token = os.getenv('CI_JOB_TOKEN')

    if args.no_release:
        publication_date = False
        release_notes = ""
        download_url = ""
    else:
        publication_date = True
        last_gitlab_release = get_gitlab_releases(api_url, project_id, token)[0]
        release_notes = last_gitlab_release['description']
        download_url = last_gitlab_release['assets']['sources'][0]['url']

    html = update_codemeta(
        codemeta_path=args.codemeta_path,
        readme_path=Path(__file__).parent.joinpath('../../README.md').resolve(),
        version=eossr_version,
        download_url=download_url,
        publication_date=publication_date,
        release_notes=release_notes,
        overwrite=True,
    )



# Try to use setuptools_scm to get the current version; this is only used
# in development installations from the git repository.
# see eossr/version.py for details
import os.path as pth

try:
    from setuptools_scm import get_version

    version = get_version(root=pth.join('..', '..'), relative_to=__file__)
except Exception:
    raise ImportError('setuptools_scm broken or not installed')
